# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from superdesk.services import BaseService
from eve.utils import ParsedRequest
from superdesk.errors import SuperdeskApiError
from superdesk import get_resource_service
from apps.content_filters.filter_condition.filter_condition import FilterCondition
from flask_babel import _


class ContentFilterService(BaseService):
    def get(self, req, lookup):
        if req is None:
            req = ParsedRequest()
        if req.args and req.args.get("is_global"):
            lookup = {"is_global": True}
        return self.backend.get(self.datasource, req=req, lookup=lookup)

    def update(self, id, updates, original):
        content_filter = dict(original)
        content_filter.update(updates)
        self._validate_no_circular_reference(content_filter, content_filter["_id"])
        super().update(id, updates, original)

    def delete(self, lookup):
        filter_id = lookup.get("_id")

        # check if the filter is referenced by any subscribers...
        subscribers = self._get_referencing_subscribers(filter_id)
        if len(subscribers) > 0:
            references = ",".join(s["name"] for s in subscribers)
            raise SuperdeskApiError.badRequestError(
                _("Content filter has been referenced by subscriber(s) {references}").format(references=references)
            )

        # check if the filter is referenced by any routing schemes...
        schemes = self._get_referencing_routing_schemes(filter_id)
        if schemes.count() > 0:
            references = ",".join(s["name"] for s in schemes)
            raise SuperdeskApiError.badRequestError(
                _("Content filter has been referenced by routing scheme(s) {references}").format(references=references)
            )

        # check if the filter is referenced by any other content filters...
        referenced_filters = self._get_content_filters_by_content_filter(filter_id)
        if referenced_filters.count() > 0:
            references = ",".join([pf["name"] for pf in referenced_filters])
            raise SuperdeskApiError.badRequestError(
                _("Content filter has been referenced in {references})").format(references=references)
            )

        return super().delete(lookup)

    def _get_content_filters_by_content_filter(self, content_filter_id):
        lookup = {"content_filter.expression.pf": {"$in": [content_filter_id]}}
        content_filters = get_resource_service("content_filters").get(req=None, lookup=lookup)
        return content_filters

    def _get_referencing_subscribers(self, filter_id):
        """Fetch all subscribers that contain a reference to the given filter.

        :param str filter_id: the referenced filter's ID
        :return: DB cursor over the results
        :rtype: :py:class:`pymongo.cursor.Cursor`
        """
        subscribers_service = get_resource_service("subscribers")
        products_service = get_resource_service("products")
        subscribers = []

        products = products_service.get(req=None, lookup={"content_filter.filter_id": filter_id})

        for p in products:
            subs = list(
                subscribers_service.get(req=None, lookup={"$or": [{"products": p["_id"]}, {"api_products": p["_id"]}]})
            )
            subscribers.extend(subs)

        return subscribers

    def _get_referencing_routing_schemes(self, filter_id):
        """Fetch all routing schemes that contain a reference to the given filter.

        :param str filter_id: the referenced filter's ID
        :return: DB cursor over the results
        :rtype: :py:class:`pymongo.cursor.Cursor`
        """
        routing_schemes_service = get_resource_service("routing_schemes")
        schemes = routing_schemes_service.get(req=None, lookup={"rules.filter": filter_id})
        return schemes

    def get_content_filters_by_filter_condition(self, filter_condition_id):
        lookup = {"content_filter.expression.fc": {"$in": [filter_condition_id]}}
        content_filters = super().get(req=None, lookup=lookup)
        all_content_filters = self._get_referenced_content_filters(list(content_filters), None)
        return all_content_filters

    def _get_referenced_content_filters(self, content_filters, pf_list):
        if not pf_list:
            pf_list = []

        for pf in content_filters:
            pf_list.append(pf)
            references = list(self._get_content_filters_by_content_filter(pf["_id"]))
            if references and len(references) > 0:
                return self._get_referenced_content_filters(references, pf_list)
        return pf_list

    def _validate_no_circular_reference(self, content_filter, filter_id):
        for expression in content_filter.get("content_filter", []):
            if "pf" in expression.get("expression", {}):
                for f in expression["expression"]["pf"]:
                    current_filter = super().find_one(req=None, _id=f)
                    if f == filter_id:
                        raise SuperdeskApiError.badRequestError(
                            _("Circular dependency error in content filters:{filter}").format(
                                filter=current_filter["name"]
                            )
                        )
                    self._validate_no_circular_reference(current_filter, filter_id)

    def build_mongo_query(self, doc):
        filter_condition_service = get_resource_service("filter_conditions")
        expressions = []
        for expression in doc.get("content_filter", []):
            filter_conditions = []
            if "fc" in expression.get("expression", {}):
                for f in expression["expression"]["fc"]:
                    current_filter = FilterCondition.parse(filter_condition_service.find_one(req=None, _id=f))
                    mongo_query = current_filter.get_mongo_query()
                    filter_conditions.append(mongo_query)
            if "pf" in expression.get("expression", {}):
                for f in expression["expression"]["pf"]:
                    current_filter = super().find_one(req=None, _id=f)
                    mongo_query = self.build_mongo_query(current_filter)
                    filter_conditions.append(mongo_query)

            if len(filter_conditions) > 1:
                expressions.append({"$and": filter_conditions})
            else:
                expressions.extend(filter_conditions)

        if len(expressions) > 1:
            return {"$or": expressions}
        else:
            return expressions[0]

    def build_elastic_query(self, doc):
        return {"query": {"filtered": {"query": self._get_elastic_query(doc)}}}

    def build_elastic_not_filter(self, doc):
        return {"query": {"filtered": {"query": self._get_elastic_query(doc, matching=False)}}}

    def _get_elastic_query(self, doc, matching=True):
        expressions_list = []
        if matching:
            expressions = {"should": expressions_list}
        else:
            expressions = {"must_not": expressions_list}

        filter_condition_service = get_resource_service("filter_conditions")
        for expression in doc.get("content_filter", []):
            filter_conditions = {"must": [], "must_not": [{"term": {"state": "spiked"}}]}
            if "fc" in expression.get("expression", {}):
                for f in expression["expression"]["fc"]:
                    current_filter = FilterCondition.parse(filter_condition_service.find_one(req=None, _id=f))
                    elastic_query = current_filter.get_elastic_query()
                    if current_filter.contains_not():
                        filter_conditions["must_not"].append(elastic_query)
                    else:
                        filter_conditions["must"].append(elastic_query)
            if "pf" in expression.get("expression", {}):
                for f in expression["expression"]["pf"]:
                    current_filter = super().find_one(req=None, _id=f)
                    elastic_query = self._get_elastic_query(current_filter)
                    filter_conditions["must"].append(elastic_query)

            expressions_list.append({"bool": filter_conditions})
        return {"bool": expressions}

    def does_match(self, content_filter, article, filters=None):
        if not content_filter:
            return True  # a non-existing filter matches every thing

        filter_condition_service = get_resource_service("filter_conditions")
        expressions = []
        for index, expression in enumerate(content_filter.get("content_filter", [])):
            if not expression.get("expression"):
                raise SuperdeskApiError.badRequestError(
                    _("Filter statement {index} does not have a filter condition").format(index=index + 1)
                )
            filter_conditions = []
            if "fc" in expression.get("expression", {}):
                for f in expression["expression"]["fc"]:
                    fc = (
                        filters.get("filter_conditions", {}).get(f, {}).get("fc")
                        if filters
                        else filter_condition_service.find_one(req=None, _id=f)
                    )
                    filter_condition = FilterCondition.parse(fc)
                    filter_conditions.append(filter_condition.does_match(article))
            if "pf" in expression.get("expression", {}):
                for f in expression["expression"]["pf"]:
                    current_filter = (
                        filters.get("content_filters", {}).get(f, {}).get("cf")
                        if filters
                        else super().find_one(req=None, _id=f)
                    )
                    filter_conditions.append(self.does_match(current_filter, article))

            expressions.append(all(filter_conditions))
        return any(expressions)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""IPTC module"""

import os
from superdesk import json
from datetime import datetime


def load_codes(filename):
    with open(filename, "r") as f:
        codes = json.load(f)
        return codes


dirname = os.path.dirname(os.path.realpath(__file__))
data_subject_codes = os.path.join(dirname, "data", "subject_codes.json")
subject_codes = load_codes(data_subject_codes)


def init_app(app):
    last_modified = datetime(2012, 7, 10)
    app.subjects.register(subject_codes, last_modified)



import random
from time import sleep

from superdesk.cache import cache
from superdesk.tests import TestCase
from bson import ObjectId


class Foo:
    def __init__(self):
        self.test_calls = 0

    @cache(ttl=1, tags=("count",))
    def count_calls(self):
        self.test_calls += 1
        return self.test_calls

    @cache(ttl=1, tags=("random",))
    def random(self):
        return random.random()

    @cache(ttl=1)
    def identity(self, identity):
        return identity


foo = Foo()


class CacheTestCase(TestCase):
    def test_cache(self):
        self.assertEqual(1, foo.count_calls())
        self.assertEqual(1, foo.count_calls(), "not using cache")
        sleep(2)
        self.assertEqual(2, foo.count_calls(), "expired")
        cache.clean()
        self.assertEqual(3, foo.count_calls(), "force expire")

        ran = foo.random()
        self.assertEqual(ran, foo.random())
        cache.clean(["random"])
        self.assertNotEqual(ran, foo.random())
        self.assertEqual(3, foo.count_calls())

    def test_cache_json_serializing(self):
        _id = ObjectId()
        self.assertEqual(_id, foo.identity(_id))
        self.assertNotEqual(_id, foo.identity(ObjectId()))
        self.assertEqual(_id, foo.identity(_id))

    def test_cache_cursor(self):
        self.app.data.insert("users", [{"username": "foo"}])

        @cache(ttl=5)
        def get_users():
            return [user for user in self.app.data.find_all("users")]

        users = get_users()
        self.assertEqual(users, get_users())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""
A module that contains exception types for the Superdesk public API.
"""


from superdesk.errors import SuperdeskApiError


class PublicApiError(SuperdeskApiError):
    """Base class for all Superdesk public API errors."""

    _codes = {
        10000: "Unknown API error.",
    }
    """A mapping of error codes to error messages."""

    def __init__(self, error_code=10000, desc=None):
        message = self._codes.get(error_code, "Unknown error")
        super().__init__(status_code=error_code, message=message, payload=desc)


class UnexpectedParameterError(PublicApiError):
    """Used when request contains an unexpected parameter."""

    PublicApiError._codes[10001] = "Unexpected parameter."

    def __init__(self, desc=None):
        super().__init__(10001, desc=desc)
        self.status_code = 400


class BadParameterValueError(PublicApiError):
    """Used when request contains a parameter with an invalid value."""

    PublicApiError._codes[10002] = "Bad parameter value."

    def __init__(self, desc=None):
        super().__init__(10002, desc=desc)
        self.status_code = 400


class FileNotFoundError(PublicApiError):
    """Used when trying to fetch a missing file."""

    PublicApiError._codes[10003] = "File not found."

    def __init__(self, desc=None):
        super().__init__(10003, desc=desc)
        self.status_code = 404



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
from eve.utils import config
from eve.versioning import versioned_id_field
from superdesk.services import BaseService
from flask import current_app as app
from superdesk.utils import ListCursor


class ItemsVersionsService(BaseService):
    def get(self, req, lookup):
        resource_def = app.config["DOMAIN"]["items"]
        id_field = versioned_id_field(resource_def)

        lookup = {"$and": [lookup, {"pubstatus": {"$ne": "canceled"}}]}
        version_history = list(super().get_from_mongo(req=req, lookup=lookup))

        for doc in version_history:
            doc[config.ID_FIELD] = doc[id_field]

        return ListCursor(version_history)

    def find_one(self, req, **lookup):
        lookup = {"$and": [lookup, {"pubstatus": {"$ne": "canceled"}}]}
        return super().find_one(req, **lookup)

    def on_item_deleted(self, document):
        """Called from ``content_api.items.ItemService`` when an item has been deleted.

        Makes sure that associated item versions are deleted along with the stored item

        :param dict document: Item that has been deleted
        """
        self.delete(lookup={"_id_document": document["_id"]})



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : sdesk
# Creation: 2016-07-22 11:16

from superdesk.commands.data_updates import BaseDataUpdate
from superdesk import get_resource_service
from eve.utils import config


class DataUpdate(BaseDataUpdate):
    """Updates the user collection with invisible stages.

    Refer to https://dev.sourcefabric.org/browse/SD-5077 for more information
    """

    resource = "users"

    def forwards(self, mongodb_collection, mongodb_database):
        for user in mongodb_collection.find({}):
            stages = get_resource_service(self.resource).get_invisible_stages_ids(user.get(config.ID_FIELD))
            print(mongodb_collection.update({"_id": user.get(config.ID_FIELD)}, {"$set": {"invisible_stages": stages}}))

    def backwards(self, mongodb_collection, mongodb_database):
        print(mongodb_collection.update({}, {"$unset": {"invisible_stages": []}}, upsert=False, multi=True))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk

from superdesk.celery_app import celery
from superdesk import get_backend, privilege
from .resource import (
    LegalArchiveResource,
    LegalArchiveVersionsResource,
    LegalPublishQueueResource,
    LegalArchiveHistoryResource,
    LEGAL_ARCHIVE_NAME,
    LEGAL_ARCHIVE_VERSIONS_NAME,
    LEGAL_ARCHIVE_HISTORY_NAME,
    LEGAL_PUBLISH_QUEUE_NAME,
)
from .service import (
    LegalArchiveService,
    LegalArchiveVersionsService,
    LegalPublishQueueService,
    LegalArchiveHistoryService,
)
from .commands import ImportLegalPublishQueueCommand, ImportLegalArchiveCommand  # noqa

logger = logging.getLogger(__name__)


def init_app(app):
    if not app.config["LEGAL_ARCHIVE"]:
        return

    endpoint_name = LEGAL_ARCHIVE_NAME
    service = LegalArchiveService(endpoint_name, backend=get_backend())
    LegalArchiveResource(endpoint_name, app=app, service=service)

    endpoint_name = LEGAL_ARCHIVE_VERSIONS_NAME
    service = LegalArchiveVersionsService(endpoint_name, backend=get_backend())
    LegalArchiveVersionsResource(endpoint_name, app=app, service=service)

    endpoint_name = LEGAL_ARCHIVE_HISTORY_NAME
    service = LegalArchiveHistoryService(endpoint_name, backend=get_backend())
    LegalArchiveHistoryResource(endpoint_name, app=app, service=service)

    endpoint_name = LEGAL_PUBLISH_QUEUE_NAME
    service = LegalPublishQueueService(endpoint_name, backend=get_backend())
    LegalPublishQueueResource(endpoint_name, app=app, service=service)

    privilege(name=LEGAL_ARCHIVE_NAME, label="Legal Archive", description="Read from legal archive")

    superdesk.command("legal_publish_queue:import", ImportLegalPublishQueueCommand())
    superdesk.command("legal_archive:import", ImportLegalArchiveCommand())


@celery.task(soft_time_limit=300)
def import_legal_publish_queue():
    ImportLegalPublishQueueCommand().run()


@celery.task(soft_time_limit=1800)
def import_legal_archive():
    ImportLegalArchiveCommand().run()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
import mimetypes
from io import BytesIO
from datetime import timedelta, datetime
from unittest.mock import patch

from bson import ObjectId
from eve.utils import ParsedRequest
from nose.tools import assert_raises

from superdesk import get_resource_service, etree
from superdesk.utc import utcnow
from superdesk.errors import SuperdeskApiError, ProviderError
from superdesk.tests import TestCase
from superdesk.tests.setup_teardown import setup_providers, teardown_providers
from superdesk.io import get_feeding_service
from superdesk.io.commands.remove_expired_content import RemoveExpiredContent, get_expired_items
from superdesk.io.feeding_services.file_service import FileFeedingService
from superdesk.io.commands.update_ingest import (
    is_scheduled,
    update_provider,
    filter_expired_items,
    apply_rule_set,
    get_task_ttl,
    get_task_id,
    get_is_idle,
    ingest_item,
)


reuters_guid = "tag_reuters.com_2014_newsml_KBN0FL0NM:10"


def get_file(url, request_kwargs=None):
    """Return file data tuple as returned by media_operations.download_file_from_url"""
    filename = os.path.basename(url)
    dirname = os.path.dirname(os.path.realpath(__file__))
    filepath = os.path.normpath(os.path.join(dirname, "fixtures", filename))
    mime = mimetypes.guess_type(filepath)[0]
    ext = str(mime).split("/")[1]
    name = str(ObjectId()) + ext
    with open(filepath, "rb") as f:
        data = f.read()
    return BytesIO(data), name, mime


class UpdateIngestTest(TestCase):
    def setUp(self):
        setup_providers(self)

    def tearDown(self):
        teardown_providers(self)

    def _get_provider(self, provider_name):
        return get_resource_service("ingest_providers").find_one(name=provider_name, req=None)

    def _get_provider_service(self, provider):
        return get_feeding_service(provider["feeding_service"])

    def setup_reuters_provider(self):
        provider_name = "reuters"
        provider = get_resource_service("ingest_providers").find_one(name=provider_name, req=None)
        provider_service = self._get_provider_service(provider)
        provider_service.provider = provider
        provider_service.URL = provider.get("config", {}).get("url")
        return provider, provider_service

    def test_ingest_items(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        items.extend(provider_service.fetch_ingest(reuters_guid))
        self.assertEqual(12, len(items))
        self.ingest_items(items, provider, provider_service)

    def test_ingest_item_expiry(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        self.assertIsNone(items[1].get("expiry"))
        items[1]["versioncreated"] = utcnow()
        self.ingest_items([items[1]], provider, provider_service)
        self.assertIsNotNone(items[1].get("expiry"))

    def test_ingest_item_sync_if_missing_from_elastic(self):
        provider, provider_service = self.setup_reuters_provider()
        item = provider_service.fetch_ingest(reuters_guid)[0]
        # insert in mongo
        ids = self.app.data._backend("ingest").insert("ingest", [item])
        # check that item is not in elastic
        elastic_item = self.app.data._search_backend("ingest").find_one("ingest", _id=ids[0], req=None)
        self.assertIsNone(elastic_item)
        # trigger sync by fetch
        old_item = get_resource_service("ingest").find_one(_id=ids[0], req=None)
        self.assertIsNotNone(old_item)
        # check that item is synced in elastic
        elastic_item = self.app.data._search_backend("ingest").find_one("ingest", _id=ids[0], req=None)
        self.assertIsNotNone(elastic_item)

    def test_ingest_provider_closed_raises_exception(self):
        provider = {
            "name": "aap",
            "is_closed": True,
            "source": "aap",
            "feeding_service": "file",
            "feed_parser": "nitf",
            "config": {"path": "/"},
        }

        with assert_raises(SuperdeskApiError) as error_context:
            aap = self._get_provider_service(provider)
            aap.update(provider, {})
        ex = error_context.exception
        self.assertTrue(ex.status_code == 500)

    def test_ingest_provider_closed_when_critical_error_raised(self):
        provider_name = "AAP"
        provider = self._get_provider(provider_name)
        self.assertFalse(provider.get("is_closed"))
        provider_service = self._get_provider_service(provider)
        provider_service.provider = provider
        provider_service.close_provider(provider, ProviderError.anpaError())
        provider = self._get_provider(provider_name)
        self.assertTrue(provider.get("is_closed"))

    def test_ingest_provider_calls_close_provider(self):
        def mock_update(provider, update):
            raise ProviderError.anpaError()

        provider_name = "AAP"
        provider = self._get_provider(provider_name)
        self.assertFalse(provider.get("is_closed"))
        provider_service = self._get_provider_service(provider)
        provider_service.provider = provider
        provider_service._update = mock_update
        with assert_raises(ProviderError):
            provider_service.update(provider, {})
        provider = self._get_provider(provider_name)
        self.assertTrue(provider.get("is_closed"))

    def test_is_scheduled(self):
        self.assertTrue(is_scheduled({}), "run after create")
        self.assertFalse(is_scheduled({"last_updated": utcnow()}), "wait default time 5m")
        self.assertTrue(is_scheduled({"last_updated": utcnow() - timedelta(minutes=6)}), "run after 5m")
        self.assertFalse(
            is_scheduled({"last_updated": utcnow() - timedelta(minutes=6), "update_schedule": {"minutes": 10}}),
            "or wait if provider has specific schedule",
        )
        self.assertTrue(
            is_scheduled({"last_updated": utcnow() - timedelta(minutes=11), "update_schedule": {"minutes": 10}}),
            "and run eventually",
        )

    def test_change_last_updated(self):
        ingest_provider = {"name": "test", "feeding_service": "file", "feed_parser": "nitf", "_etag": "test"}
        self.app.data.insert("ingest_providers", [ingest_provider])

        update_provider(ingest_provider)
        provider = self.app.data.find_one("ingest_providers", req=None, _id=ingest_provider["_id"])
        self.assertGreaterEqual(utcnow(), provider.get("last_updated"))
        self.assertEqual("test", provider.get("_etag"))

    def test_filter_expired_items(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        for item in items[:4]:
            item["expiry"] = utcnow() + timedelta(minutes=11)
        self.assertEqual(4, len(filter_expired_items(provider, items)))

    def test_filter_expired_items_with_no_expiry(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        self.assertEqual(0, len(filter_expired_items(provider, items)))

    def test_query_getting_expired_content(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        now = utcnow()
        for i, item in enumerate(items):
            item["ingest_provider"] = provider["_id"]
            expiry_time = now - timedelta(hours=11)
            if i > 4:
                expiry_time = now + timedelta(minutes=11)

            item["expiry"] = item["versioncreated"] = expiry_time

        service = get_resource_service("ingest")
        service.post(items)
        expiredItems = get_expired_items(provider, "ingest")
        self.assertEqual(5, expiredItems.count())

    def test_expiring_with_content(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        now = utcnow()
        for i, item in enumerate(items):
            item["ingest_provider"] = provider["_id"]
            expiry_time = now - timedelta(hours=11)
            if i > 4:
                expiry_time = now + timedelta(minutes=11)

            item["expiry"] = item["versioncreated"] = expiry_time

        service = get_resource_service("ingest")
        service.post(items)

        # ingest the items and expire them
        before = service.get(req=None, lookup={})
        self.assertEqual(6, before.count())

        remove = RemoveExpiredContent()
        remove.run(provider.get("type"))

        # only one left in ingest
        after = service.get(req=None, lookup={})
        self.assertEqual(1, after.count())

        req = ParsedRequest()
        self.assertEqual(1, self.app.data.elastic.find("ingest", req, {}).count())
        self.assertEqual(1, self.app.data.mongo.find("ingest", req, {})[1])

    def test_removing_expired_items_from_elastic_only(self):
        now = utcnow()
        self.app.data.elastic.insert(
            "ingest",
            [
                {"_id": "foo", "expiry": now - timedelta(minutes=30)},
                {"_id": "bar", "expiry": now + timedelta(minutes=30)},
            ],
        )

        RemoveExpiredContent().run()
        self.assertEqual(1, self.app.data.elastic.find("ingest", ParsedRequest(), {}).count())

    def test_expiring_content_with_files(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        for item in items:
            item["ingest_provider"] = provider["_id"]

        now = utcnow()
        items[0]["expiry"] = now - timedelta(hours=11)
        items[1]["expiry"] = now - timedelta(hours=11)
        items[2]["expiry"] = now + timedelta(hours=11)
        items[5]["versioncreated"] = now + timedelta(minutes=11)

        # ingest the items and expire them
        self.ingest_items(items, provider, provider_service)

        # four files in grid fs
        current_files = self.app.media.storage().fs("upload").find()
        self.assertEqual(4, current_files.count())

        remove = RemoveExpiredContent()
        remove.run(provider.get("type"))

        # all gone
        current_files = self.app.media.storage().fs("upload").find()
        self.assertEqual(0, current_files.count())

    def test_apply_rule_set(self):
        item = {"body_html": "@@body@@"}

        provider_name = "reuters"
        provider = self._get_provider(provider_name)
        self.assertEqual("body", apply_rule_set(item, provider)["body_html"])

        item = {"body_html": "@@body@@"}
        provider_name = "AAP"
        provider = self._get_provider(provider_name)
        self.assertEqual("@@body@@", apply_rule_set(item, provider)["body_html"])

    def test_all_ingested_items_have_sequence(self):
        provider, provider_service = self.setup_reuters_provider()
        guid = "tag_reuters.com_2014_newsml_KBN0FL0NM:10"
        item = provider_service.fetch_ingest(guid)[0]
        get_resource_service("ingest").set_ingest_provider_sequence(item, provider)
        self.assertIsNotNone(item["ingest_provider_sequence"])

    def test_get_task_ttl(self):
        self.assertEqual(300, get_task_ttl({}))
        provider = {"update_schedule": {"minutes": 10}}
        self.assertEqual(600, get_task_ttl(provider))
        provider["update_schedule"]["hours"] = 1
        provider["update_schedule"]["minutes"] = 1
        self.assertEqual(3660, get_task_ttl(provider))

    def test_get_task_id(self):
        provider = {"name": "foo", "_id": "abc"}
        self.assertEqual("update-ingest-foo-abc", get_task_id(provider))

    def test_is_idle(self):
        provider = dict(idle_time=dict(hours=1, minutes=0))
        provider["last_item_update"] = utcnow()
        self.assertEqual(get_is_idle(provider), False)
        provider["idle_time"]["hours"] = -1
        self.assertEqual(get_is_idle(provider), True)
        provider["idle_time"] = dict(hours=0, minutes=0)
        self.assertEqual(get_is_idle(provider), False)

    def test_files_dont_duplicate_ingest(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)

        for item in items:
            item["ingest_provider"] = provider["_id"]
            item["expiry"] = utcnow() + timedelta(hours=11)

        # ingest the items
        self.ingest_items(items, provider, provider_service)

        items = provider_service.fetch_ingest(reuters_guid)
        for item in items:
            item["ingest_provider"] = provider["_id"]
            item["expiry"] = utcnow() + timedelta(hours=11)

        # ingest them again
        self.ingest_items(items, provider, provider_service)

        # 12 files in grid fs
        current_files = self.app.media.storage().fs("upload").find()
        self.assertEqual(12, current_files.count())

    def test_anpa_category_to_subject_derived_ingest(self):
        vocab = [
            {
                "_id": "categories",
                "items": [{"is_active": True, "name": "Domestic Sport", "qcode": "s", "subject": "15000000"}],
            }
        ]
        self.app.data.insert("vocabularies", vocab)

        provider_name = "DPA"
        provider = get_resource_service("ingest_providers").find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get("config", {}).get("path", ""), "IPTC7901_odd_charset.txt")
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        items = [feeding_parser.parse(file_path, provider)]

        # ingest the items and check the subject code has been derived
        self.ingest_items(items, provider, provider_service)
        self.assertEqual(items[0]["subject"][0]["qcode"], "15000000")

    def test_anpa_category_to_subject_derived_ingest_ignores_inactive_categories(self):
        vocab = [
            {
                "_id": "categories",
                "items": [{"is_active": False, "name": "Domestic Sport", "qcode": "s", "subject": "15000000"}],
            }
        ]
        self.app.data.insert("vocabularies", vocab)

        provider_name = "DPA"
        provider = get_resource_service("ingest_providers").find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get("config", {}).get("path", ""), "IPTC7901_odd_charset.txt")
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        items = [feeding_parser.parse(file_path, provider)]

        # ingest the items and check the subject code has been derived
        self.ingest_items(items, provider, provider_service)
        self.assertNotIn("subject", items[0])

    def test_subject_to_anpa_category_derived_ingest(self):
        vocab = [
            {
                "_id": "iptc_category_map",
                "items": [{"name": "Finance", "category": "f", "qcode": "04000000", "is_active": True}],
            },
            {
                "_id": "categories",
                "items": [
                    {"is_active": True, "name": "Australian Weather", "qcode": "b", "subject": "17000000"},
                    {"is_active": True, "name": "Finance", "qcode": "f"},
                ],
            },
            {"_id": "genre", "items": [{"qcode": "feature", "name": "feature"}]},
        ]

        self.app.data.insert("vocabularies", vocab)

        provider_name = "AAP"
        provider = get_resource_service("ingest_providers").find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get("config", {}).get("path", ""), "nitf-fishing.xml")
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        with open(file_path, "r") as f:
            xml_string = etree.etree.fromstring(f.read())
            items = [feeding_parser.parse(xml_string, provider)]
            for item in items:
                item["ingest_provider"] = provider["_id"]
                item["expiry"] = utcnow() + timedelta(hours=11)

            # ingest the items and check the subject code has been derived
            self.ingest_items(items, provider, provider_service)
            self.assertEqual(items[0]["anpa_category"][0]["qcode"], "f")

    def test_subject_to_anpa_category_derived_ingest_ignores_inactive_map_entries(self):
        vocab = [
            {
                "_id": "iptc_category_map",
                "items": [{"name": "Finance", "category": "f", "qcode": "04000000", "is_active": False}],
            },
            {
                "_id": "categories",
                "items": [{"is_active": True, "name": "Australian Weather", "qcode": "b", "subject": "17000000"}],
            },
            {"_id": "genre", "items": [{"qcode": "feature", "name": "feature"}]},
        ]

        self.app.data.insert("vocabularies", vocab)

        provider_name = "AAP"
        provider = get_resource_service("ingest_providers").find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get("config", {}).get("path", ""), "nitf-fishing.xml")
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        with open(file_path, "r") as f:
            xml_string = etree.etree.fromstring(f.read())
            items = [feeding_parser.parse(xml_string, provider)]
            for item in items:
                item["ingest_provider"] = provider["_id"]
                item["expiry"] = utcnow() + timedelta(hours=11)

            # ingest the items and check the subject code has been derived
            self.ingest_items(items, provider, provider_service)
            self.assertNotIn("anpa_category", items[0])

    def test_ingest_cancellation(self):
        provider, provider_service = self.setup_reuters_provider()
        guid = "tag_reuters.com_2016_newsml_L1N14N0FF:978556838"
        items = provider_service.fetch_ingest(guid)
        for item in items:
            item["ingest_provider"] = provider["_id"]
            item["expiry"] = utcnow() + timedelta(hours=11)
        self.ingest_items(items, provider, provider_service)
        guid = "tag_reuters.com_2016_newsml_L1N14N0FF:1542761538"
        items = provider_service.fetch_ingest(guid)
        for item in items:
            item["ingest_provider"] = provider["_id"]
            item["expiry"] = utcnow() + timedelta(hours=11)
        self.ingest_items(items, provider, provider_service)
        ingest_service = get_resource_service("ingest")
        lookup = {"uri": items[0].get("uri")}
        family_members = ingest_service.get_from_mongo(req=None, lookup=lookup)
        self.assertEqual(family_members.count(), 2)
        for relative in family_members:
            self.assertEqual(relative["pubstatus"], "canceled")
            self.assertEqual(relative["state"], "killed")

    def test_ingest_update(self):
        provider, provider_service = self.setup_reuters_provider()
        items = provider_service.fetch_ingest(reuters_guid)
        items[0]["ingest_provider"] = provider["_id"]
        items[0]["expiry"] = utcnow() + timedelta(hours=11)

        self.ingest_items(items, provider, provider_service)

        self.assertEqual(items[0]["unique_id"], 1)
        original_id = items[0]["_id"]

        items = provider_service.fetch_ingest(reuters_guid)
        items[0]["ingest_provider"] = provider["_id"]
        items[0]["expiry"] = utcnow() + timedelta(hours=11)
        # change the headline
        items[0]["headline"] = "Updated headline"

        # ingest the item again
        self.ingest_items(items, provider, provider_service)

        # see the update to the headline and unique_id survives
        elastic_item = self.app.data._search_backend("ingest").find_one("ingest", _id=original_id, req=None)
        self.assertEqual(elastic_item["headline"], "Updated headline")
        self.assertEqual(elastic_item["unique_id"], 1)
        self.assertEqual(elastic_item["unique_name"], "#1")

    def test_get_article_ids(self):
        provider_name = "reuters"
        provider, provider_service = self.setup_reuters_provider()
        ids = provider_service._get_article_ids("channel1", utcnow(), utcnow() + timedelta(minutes=-10))
        self.assertEqual(len(ids), 3)
        provider = get_resource_service("ingest_providers").find_one(name=provider_name, req=None)
        self.assertEqual(provider["tokens"]["poll_tokens"]["channel1"], "ExwaY31kfnR2Z2J1cWZ2YnxoYH9kfw==")

    def test_unknown_category_ingested_is_removed(self):
        vocab = [
            {
                "_id": "categories",
                "items": [
                    {"is_active": True, "name": "Australian Weather", "qcode": "b", "subject": "17000000"},
                    {"is_active": True, "name": "Finance", "qcode": "f"},
                ],
            }
        ]

        self.app.data.insert("vocabularies", vocab)

        provider_name = "AP"
        provider = get_resource_service("ingest_providers").find_one(name=provider_name, req=None)
        file_path = os.path.join(provider.get("config", {}).get("path", ""), "ap_anpa-3.tst")
        provider_service = self._get_provider_service(provider)
        feeding_parser = provider_service.get_feed_parser(provider)
        items = [feeding_parser.parse(file_path, provider)]

        # ingest the items and check the subject code has been derived
        items[0]["versioncreated"] = utcnow()
        self.ingest_items(items, provider, provider_service)
        self.assertTrue(len(items[0]["anpa_category"]) == 0)

    def test_ingest_with_routing_keeps_elastic_in_sync(self):
        provider, provider_service = self.setup_reuters_provider()

        desk = {"name": "foo"}
        self.app.data.insert("desks", [desk])
        self.assertIsNotNone(desk["_id"])
        self.assertIsNotNone(desk["incoming_stage"])

        now = datetime.now()

        items = [
            {
                "guid": "main_text",
                "versioncreated": now,
                "headline": "Headline of the text item",
            },
            {"guid": "image_1", "type": "picture", "versioncreated": now},
            {"guid": "image_2", "type": "picture", "versioncreated": now},
            {
                "type": "composite",
                "guid": "package:guid:abcd123",
                "versioncreated": now,
                "headline": "Headline of the text item",
                "groups": [
                    {
                        "id": "root",
                        "role": "grpRole:NEP",
                        "refs": [{"idRef": "main"}],
                    },
                    {
                        "id": "main",
                        "role": "main",
                        "refs": [
                            {"residRef": "main_text"},
                            {"residRef": "image_1"},
                            {"residRef": "image_2"},
                        ],
                    },
                ],
            },
        ]

        all_week_schedule = {
            "day_of_week": ["MON", "TUE", "WED", "THU", "FRI", "SAT", "SUN"],
            "hour_of_day_from": "00:00:00",
            "hour_of_day_to": "23:55:00",
            "time_zone": "Europe/Prague",
        }

        routing_scheme = {
            "name": "autofetch",
            "rules": [
                {
                    "filter": None,
                    "actions": {
                        "exit": False,
                        "publish": [],
                        "fetch": [{"stage": desk["incoming_stage"], "desk": desk["_id"]}],
                    },
                    "schedule": all_week_schedule,
                    "name": "fetch",
                },
                {
                    "filter": None,
                    "actions": {"exit": False, "publish": [], "fetch": []},
                    "schedule": all_week_schedule,
                    "name": "empty",
                },
            ],
        }

        ingest_service = get_resource_service("ingest")
        self.ingest_items(items, provider, provider_service, routing_scheme=routing_scheme)

        self.assertEqual(4, ingest_service.get_from_mongo(None, {}).count())
        self.assertEqual(4, ingest_service.get(None, {}).count())

        for item in items:
            lookup = {"guid": item["guid"]}
            mongo_item = ingest_service.get_from_mongo(None, lookup)[0]
            elastic_item = ingest_service.get(None, lookup)[0]
            self.assertEqual(mongo_item["_etag"], elastic_item["_etag"], mongo_item["guid"])

    @patch("superdesk.media.renditions.download_file_from_url", get_file)
    def test_ingest_associated_item_renditions(self):
        provider = {"feeding_service": "ninjs", "_id": self.providers["ninjs"]}
        provider_service = FileFeedingService()
        item = {
            "guid": "foo",
            "type": "text",
            "versioncreated": utcnow(),
            "associations": {
                "featuremedia": {
                    "guid": "bar",
                    "type": "picture",
                    "versioncreated": utcnow(),
                    "renditions": {
                        "original": {
                            "href": "https://farm8.staticflickr.com/7300/picture_bug.jpg",
                            "mimetype": "image/jpeg",
                            "width": 640,
                            "height": 426,
                        }
                    },
                },
                "foo": {
                    "guid": "foo",
                    "type": "picture",
                    "versioncreated": utcnow(),
                    "renditions": {
                        "original": {
                            "href": "https://farm8.staticflickr.com/7300/picture_bug.jpg",
                            "mimetype": "image/jpeg",
                            "width": 640,
                            "height": 426,
                        },
                    },
                },
            },
        }

        # avoid transfer_renditions call which would store the picture locally
        # and it would fetch it using superdesk url which doesn't work in test
        with patch("superdesk.io.commands.update_ingest.transfer_renditions"):
            status, ids = ingest_item(item, provider, provider_service)

        self.assertTrue(status)
        self.assertEqual(3, len(ids))
        self.assertIn("thumbnail", item["associations"]["featuremedia"]["renditions"])
        self.assertIn("thumbnail", item["associations"]["foo"]["renditions"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import datetime
import os
import unittest

from superdesk.etree import etree
from superdesk.io.feed_parsers.wenn_parser import WENNFeedParser
from superdesk.utc import utc


class WENNTestCase(unittest.TestCase):
    filename = "wenn.xml"

    def setUp(self):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Wenn"}
        with open(fixture, "rb") as f:
            self.file = f.read()
            etree.fromstring(self.file)
            self.items = WENNFeedParser().parse(etree.fromstring(self.file), provider)

    def test_items_counts(self):
        self.assertEqual(len(self.items), 2)

    def test_headline(self):
        self.assertEqual(self.items[0].get("headline"), "Suge Knight involved in fatal hit and run incident")
        self.assertEqual(self.items[1].get("headline"), "Jagged Edge singer Kyle Norman arrested over domestic assault")

    def test_abstract(self):
        self.assertEqual(
            self.items[0].get("abstract"),
            "Police officials in Los Angeles are investigating a fatal hit and run incident involving "
            "rap mogul SUGE KNIGHT on Thursday (29Jan15).",
        )
        self.assertEqual(
            self.items[1].get("abstract"),
            "JAGGED EDGE singer KYLE NORMAN has been arrested by police in Atlanta, Georgia amid"
            " allegations of aggravated assault.",
        )

    def test_anpa_category(self):
        self.assertEqual(self.items[0].get("anpa_category")[0]["qcode"], "e")
        self.assertEqual(self.items[1].get("anpa_category")[0]["qcode"], "e")

    def test_subject(self):
        self.assertEqual(self.items[0].get("subject")[0]["qcode"], "01000000")
        self.assertEqual(self.items[1].get("subject")[0]["qcode"], "01000000")

    def test_firstcreated(self):
        self.assertEqual(
            self.items[0].get("firstcreated"),
            datetime.datetime(year=2015, month=1, day=30, hour=0, minute=33, second=6, tzinfo=utc),
        )

    def test_versioncreated(self):
        self.assertEqual(
            self.items[0].get("versioncreated"),
            datetime.datetime(year=2015, month=1, day=30, hour=2, minute=40, second=56, tzinfo=utc),
        )

    def test_body(self):
        self.assertEqual(self.items[0].get("body_html"), "<p>This is body content1.</p>")
        self.assertEqual(self.items[1].get("body_html"), "<p>This is body content2.</p>")

    def test_item_defaults(self):
        self.assertEqual(self.items[0].get("pubstatus"), "usable")
        self.assertEqual(self.items[0].get("urgency"), 5)
        self.assertEqual(self.items[0].get("type"), "text")

    def test_guid(self):
        self.assertEqual(self.items[0].get("guid"), "1369426")
        self.assertEqual(self.items[1].get("guid"), "1369417")



import json
import string
import random

from eve.utils import ParsedRequest
from eve.io.mongo import MongoJSONEncoder

from apps.dictionaries.service import words, DictionaryService, store_dict, fetch_dict, FILE_ID
from superdesk import get_resource_service
from superdesk.tests import TestCase


class WordsTestCase(TestCase):
    def setUp(self):
        self.req = ParsedRequest()
        with self.app.test_request_context(self.app.config.get("URL_PREFIX")):
            self.dictionaries = [
                {"_id": "1", "name": "Eng", "language_id": "en"},
                {"_id": "2", "name": "Eng AUs", "language_id": "en-AU", "is_active": "true"},
                {"_id": "3", "name": "French", "language_id": "fr"},
            ]
            self.app.data.insert("dictionaries", self.dictionaries)

    def test_words_parsing(self):
        self.assertEquals(["abc"], words("abc"))
        self.assertEqual(["abc bce"], words("abc bce"))
        self.assertEqual(["abc bce", "wer tyu"], words("abc bce\nwer tyu"))
        self.assertEqual(["you'd"], words("you'd"))
        self.assertEqual(["you'd", "will"], words("you'd\nwill"))

    def test_base_language(self):
        self.assertEqual(DictionaryService().get_base_language("en-AU"), "en")
        self.assertIsNone(DictionaryService().get_base_language("en"))

    def test_get_dictionary(self):
        with self.app.app_context():
            dicts = get_resource_service("dictionaries").get_dictionaries("en")
            self.assertEqual(len(dicts), 1)
            self.assertEqual(dicts[0]["language_id"], "en")
            dicts = get_resource_service("dictionaries").get_dictionaries("en-AU")
            self.assertEqual(len(dicts), 1)
            self.assertEqual(dicts[0]["language_id"], "en-AU")

    def test_store_dict_small(self):
        content = {"foo": 1}
        dictionary = {"content": content}
        store_dict(dictionary, {})
        self.assertEqual(json.dumps(content), dictionary.get("content"))
        self.assertEqual(content, fetch_dict(dictionary))

    def test_store_dict_big(self):
        content = self._get_big_dict()
        dictionary = {"content": content}
        self.assertGreater(len(json.dumps(dictionary, cls=MongoJSONEncoder)), 1000000)
        store_dict(dictionary, {})
        self.assertIsNone(dictionary["content"])
        self.assertEqual(content, fetch_dict(dictionary))
        self.assertLess(len(json.dumps(dictionary, cls=MongoJSONEncoder)), 100)

    def test_store_big_after_small(self):
        original = {"content": {"foo": 1}}
        updates = {"content": self._get_big_dict()}
        store_dict(updates, original)
        self.assertIsNone(updates["content"])
        self.assertIsNotNone(updates[FILE_ID])

    def test_small_after_big(self):
        updates = {"content": self._get_big_dict()}
        store_dict(updates, {})
        file_id = updates[FILE_ID]
        self.assertIsNotNone(self.app.storage.get(file_id))

        original = updates
        updates = {"content": {"foo": 1}}
        store_dict(updates, original)

        self.assertIsNone(updates[FILE_ID])
        self.assertEqual(json.dumps({"foo": 1}), updates["content"])
        self.assertIsNone(self.app.storage.get(file_id))

    def _get_big_dict(self):
        word = "".join([random.choice(string.ascii_letters) for i in range(1000000)])
        return {word: 1}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.validate.validate import SchemaValidator, ValidateService
from superdesk.tests import TestCase
from superdesk.metadata.item import ITEM_TYPE
from superdesk.default_settings import VALIDATOR_MEDIA_METADATA


# dict of fields mandatory in media during validation
# this is build dynamically to stay in sync with VALIDATOR_MEDIA_METADATA
MEDIA_MANDATORY = {k: k for k, v in VALIDATOR_MEDIA_METADATA.items() if v.get("required")}


class ValidateMandatoryInListTest(TestCase):
    def test_fail_validate_mandatory_in_list_for_subject(self):
        mandatory = {"scheme": {"custom_subject": {"required": True}, "category": {"required": True}}}
        value = [{"name": "DiDFødselsdag", "qcode": "DiDFødselsdag", "scheme": "category", "service": {"d": 1, "i": 1}}]
        doc = {"subject": value}
        schema = {"subject": {"type": "list", "mandatory_in_list": mandatory}}
        validator = SchemaValidator(schema)
        self.assertFalse(validator.validate(doc))
        self.assertIn("is a required field", validator.errors["custom_subject"])

    def test_fail_validate_mandatory_in_list_for_subject_and_category(self):
        mandatory = {"scheme": {"custom_subject": {"required": True}, "category": {"required": True}}}
        doc = {"subject": []}
        schema = {"subject": {"type": "list", "mandatory_in_list": mandatory}}
        validator = SchemaValidator(schema)
        self.assertFalse(validator.validate(doc))
        self.assertIn("is a required field", validator.errors["custom_subject"])
        self.assertIn("is a required field", validator.errors["category"])

    def test_validate_mandatory_in_list(self):
        validator = SchemaValidator()
        mandatory = {"scheme": {"custom_subject": {"required": True}, "category": {"required": True}}}
        field = "scheme"
        value = [
            {"name": "DiDFødselsdag", "qcode": "DiDFødselsdag", "scheme": "category", "service": {"d": 1, "i": 1}},
            {"name": "arkeologi", "qcode": "01001000", "scheme": "custom_subject", "parent": "01000000"},
        ]
        validator._validate_mandatory_in_list(mandatory, field, value)

        self.assertEqual(validator.errors, {})

    def test_sanitize_fields_not_in_schema(self):
        doc = {"body_html": "test"}
        service = ValidateService()
        schema = {"schema": {"body_html": None}}
        service._sanitize_fields(doc, schema)
        self.assertEqual("test", doc["body_html"])

    def test_validate_date_with_success(self):
        validator = SchemaValidator({"test": {"type": "date"}})
        print("mapping", validator.types_mapping)
        self.assertTrue(validator.validate({"test": "2017-11-22T22:11:33+0000"}))

    def test_validate_date_with_error(self):
        validator = SchemaValidator({"test": {"type": "date"}})
        self.assertFalse(validator.validate({"test": "2017-11-33T22:11:33+0000"}))
        self.assertIn("must be of date type", validator.errors["test"])

    def test_validate_field_without_schema(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "headline": {"required": True},
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "foo"},
                },
            ],
            fields=True,
        )
        self.assertIn("HEADLINE is a required field", errors[0][0])

    def test_validate_required_empty_string(self):
        self.app.data.insert("content_types", [{"_id": "foo", "schema": {"headline": {"required": True}}}])

        service = ValidateService()
        errors = service.create(
            [{"act": "test", "type": "test", "validate": {"profile": "foo", "headline": ""}}], fields=True
        )

        self.assertEqual(["HEADLINE empty values not allowed"], errors[0][0])

    def test_validate_required_empty_list(self):
        self.app.data.insert(
            "content_types", [{"_id": "foo", "schema": {"subject": {"type": "list", "required": True}}}]
        )

        service = ValidateService()
        errors = service.create(
            [{"act": "test", "type": "test", "validate": {"profile": "foo", "subject": []}}], fields=True
        )

        self.assertEqual(["SUBJECT is a required field"], errors[0][0])

    def test_validate_required_subject_with_cv(self):
        """Test that subject required error is raised as expected when a custom vocabulary is used"""
        self.app.data.insert(
            "content_types", [{"_id": "foo", "schema": {"subject": {"type": "list", "required": True}}}]
        )

        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {
                        "profile": "foo",
                        "subject": [{"qcode": "test", "name": "test", "scheme": "custom_cv"}],
                    },
                }
            ],
            fields=True,
        )

        self.assertEqual(["SUBJECT is a required field"], errors[0][0])

    def test_validate_required_none_list(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "subject": {
                            "type": "list",
                            "required": True,
                            "mandatory_in_list": {
                                "scheme": {"custom_subject": {"required": True}, "category": {"required": True}}
                            },
                            "schema": {
                                "type": "dict",
                                "schema": {
                                    "name": {},
                                    "qcode": {},
                                    "scheme": {
                                        "type": "string",
                                        "required": True,
                                        "allowed": ["subject_custom", "category"],
                                    },
                                    "service": {"nullable": True},
                                    "parent": {"nullable": True},
                                },
                            },
                        }
                    },
                }
            ],
        )

        service = ValidateService()
        errors = service.create(
            [{"act": "test", "type": "test", "validate": {"profile": "foo", "subject": None}}], fields=True
        )

        self.assertIn("CATEGORY is a required field", errors[0][0])
        self.assertIn("SUBJECT is a required field", errors[0][0])

    def test_validate_field_required_feature_media(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "feature_media": {"required": True},
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "foo"},
                },
            ],
            fields=True,
        )
        self.assertEqual(["FEATURE MEDIA is a required field"], errors[0][0])

    def test_validate_field_required_media_description_empty(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "feature_media": {"required": True, "type": "media"},
                        "media_description": {"required": True},
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "foo", "associations": {"featuremedia": {}}},
                }
            ],
            fields=True,
        )
        print(errors[0][0])
        self.assertIn("MEDIA DESCRIPTION is a required field", errors[0][0])

    def test_validate_field_required_media_description_null(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "feature_media": {"required": True},
                        "media_description": {"required": True},
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "foo", "associations": {"featuremedia": None}},
                },
            ],
            fields=True,
        )
        self.assertIn("FEATURE MEDIA is a required field", errors[0][0])
        self.assertIn("MEDIA DESCRIPTION is a required field", errors[0][0])

    def test_validate_field_required_media_description_required_false(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "feature_media": {"required": True, "type": "media"},
                        "media_description": {"required": False},
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "foo", "associations": {"featuremedia": None}},
                },
            ],
            fields=True,
        )
        self.assertIn("FEATURE MEDIA is a required field", errors[0][0])

    def test_validate_field_required_media_description_required_false_null_true(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "feature_media": {"required": False, "nullable": True, "type": "media"},
                        "media_description": {"required": False, "nullable": True},
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "foo", "associations": {"featuremedia": None}},
                },
            ],
            fields=True,
        )
        self.assertEqual([], errors[0][0])

    def test_validate_field_feature_media_and_media_description(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "feature_media": {"required": True, "type": "media"},
                        "media_description": {"required": True},
                    },
                }
            ],
        )
        service = ValidateService()
        feature_media = MEDIA_MANDATORY
        feature_media.update({"description_text": "test"})
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "foo", "associations": {"featuremedia": feature_media}},
                },
            ],
            fields=True,
        )
        self.assertEqual([], errors[0][0])

    def test_validate_custom_fields(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "embed1": {"required": True, "enabled": True, "type": "embed", "nullable": False},
                        "text1": {
                            "minlength": 10,
                            "required": True,
                            "enabled": True,
                            "type": "text",
                            "maxlength": 160,
                            "nullable": False,
                        },
                        "date1": {"required": True, "enabled": True, "type": "date", "nullable": False},
                    },
                }
            ],
        )
        self.app.data.insert(
            "vocabularies",
            [
                {"_id": "embed1", "type": "manageable", "field_type": "embed"},
                {"_id": "text1", "type": "manageable", "field_type": "text"},
                {"_id": "date1", "type": "manageable", "field_type": "date"},
            ],
        )
        service = ValidateService()
        doc = {"act": "test", ITEM_TYPE: "test", "validate": {"profile": "foo"}}
        schema = {
            "extra": {
                "schema": {
                    "embed1": {"required": True, "enabled": True, "nullable": False, "empty": False, "type": "dict"},
                    "text1": {
                        "enabled": True,
                        "required": True,
                        "nullable": False,
                        "minlength": 10,
                        "maxlength": 160,
                        "type": "string",
                    },
                    "date1": {"required": True, "enabled": True, "nullable": False, "empty": False, "type": "date"},
                },
                "type": "dict",
            }
        }
        self.assertEqual(service._get_validators(doc)[0]["schema"], schema)

    def test_validate_field_required_related_content_error(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "related_content_field": {"required": True, "type": "related_content"},
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "foo", "associations": {}},
                },
            ],
            fields=True,
        )
        self.assertIn("RELATED_CONTENT_FIELD is a required field", errors[0][0])

    def test_validate_field_required_related_content(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "slugline": None,
                        "related_content_field": {"required": True, "type": "related_content"},
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {
                        "profile": "foo",
                        "slugline": "foo",
                        "associations": {"related_content_field--1": MEDIA_MANDATORY},
                    },
                },
            ],
            fields=True,
        )
        self.assertEqual([], errors[0][0])

    def test_sanitize_text_fields(self):
        item = {"headline": "<p>headline</p>", "extra": {"text1": "<p>text 1</p>"}}
        sanitized_item = {"headline": "headline", "extra": {"text1": "text 1"}}
        validator = {
            "schema": {
                "headline": {"maxlength": 64, "nullable": True, "required": False, "type": "string"},
                "extra": {
                    "schema": {"text1": {"maxlength": 10, "nullable": True, "required": False, "type": "string"}}
                },
            }
        }
        ValidateService()._sanitize_fields(item, validator)
        self.assertEqual(item, sanitized_item)

    def test_validate_field_sms(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "sms": {
                            "minlength": 10,
                            "required": True,
                            "enabled": True,
                            "type": "string",
                            "maxlength": 160,
                            "nullable": True,
                        }
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "flags": {"marked_for_sms": True}, "sms_message": "short"},
                },
            ],
            fields=True,
        )
        self.assertIn("SMS is too short", errors[0][0])

    def test_validate_field_sms_not_enabled(self):
        self.app.data.insert(
            "content_types",
            [
                {
                    "_id": "foo",
                    "schema": {
                        "sms": {
                            "minlength": 10,
                            "required": True,
                            "enabled": True,
                            "type": "string",
                            "maxlength": 160,
                            "nullable": True,
                        }
                    },
                }
            ],
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "flags": {"marked_for_sms": False}, "sms_message": "short"},
                },
            ],
            fields=True,
        )
        self.assertEqual([], errors[0][0])

    def test_validate_process_media(self):
        media = {"headline": "media 1"}
        item = {"associations": {"media1--1": media}}
        validation_schema = {"media1": {"required": True}}
        ValidateService()._process_media(item, validation_schema)
        self.assertIn("media1", item)
        self.assertEqual(media, item["media1"])

    def test_validate_validate_characters(self):
        self.app.config["DISALLOWED_CHARACTERS"] = ["!", "@", "#"]
        self.app.data.insert(
            "content_types", [{"_id": "foo", "schema": {"slugline": {"validate_characters": True, "type": "string"}}}]
        )
        service = ValidateService()
        errors = service.create(
            [
                {
                    "act": "test",
                    "type": "test",
                    "validate": {"profile": "foo", "slugline": "!foo@#"},
                },
            ],
            fields=True,
        )
        self.assertIn("SLUGLINE contains invalid characters", errors[0][0])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from .workspace import WorkspaceService, WorkspaceResource


def init_app(app):
    superdesk.register_resource(
        "workspaces", WorkspaceResource, WorkspaceService, privilege=["POST", "PATCH", "DELETE"]
    )

    superdesk.register_default_user_preference("workspace:active", {"type": "string", "workspace": ""})

    superdesk.privilege(name="dashboard", label="Dashboard", description="Can access dashboard.")



import os
import re
import socket
import logging

from datetime import datetime
from mongolock import MongoLock, MongoLockException
from werkzeug.local import LocalProxy
from flask import current_app as app
from superdesk.logging import logger
from superdesk.utc import utcnow


_lock_resource_settings = {
    "internal_resource": True,
    "versioning": False,
}

logger = logging.getLogger(__name__)


class SuperdeskMongoLock(MongoLock):
    """Superdesk MongoLock

    Change release to remove the lock instead of updating it
    so there is no need to gc that collection.
    """

    def release(self, key, owner, remove=False):
        if remove:
            return self.collection.delete_one({"_id": key, "owner": owner})
        else:
            return super().release(key, owner)

    def _try_get_lock(self, key, owner, expire):
        """Log warning in case lock is gained after expiry.

        This should not happen in general, locks should be released.
        Consider increasing lock time.
        """
        lock_info = self.get_lock_info(key)
        locked = super()._try_get_lock(key, owner, expire)
        if locked and lock_info and lock_info["locked"]:
            logger.warning("Lock %s expired", key)
        return locked


def _get_lock():
    """Get mongolock instance using app mongodb."""
    app.register_resource("_lock", _lock_resource_settings)  # setup dummy resource for locks
    return SuperdeskMongoLock(client=app.data.mongo.pymongo("_lock").db)


_lock = LocalProxy(_get_lock)


def get_host():
    return "hostid:{} pid:{}".format(socket.gethostname(), os.getpid())


def lock(task, host=None, expire=300, timeout=None):
    """Try to lock task.

    :param task: task name
    :param host: current host id
    :param expire: lock ttl in seconds
    :param timeout: how long should it wait if task is locked
    """
    if not host:
        host = get_host()
    got_lock = _lock.lock(task, host, expire=expire, timeout=timeout)
    if got_lock:
        logger.debug("got lock task=%s host=%s" % (task, host))
    else:
        logger.debug("task locked already task=%s host=%s" % (task, host))
    return got_lock


def unlock(task, host=None, remove=False):
    """Release lock on given task.

    Lock can be only released by host which locked it.

    :param task: task name
    :param host: current host id
    :param remove: remove lock when unlocking
    """
    if not host:
        host = get_host()
    logger.debug("releasing lock task=%s host=%s" % (task, host))
    return _lock.release(task, host, remove)


def remove_locks():
    """
    Removes item related locks that are not in use
    :return:
    """
    result = _lock.collection.delete_many(
        {
            "$or": [
                {"_id": re.compile("^item_move"), "locked": False},
                {"_id": re.compile("^item_lock"), "locked": False},
            ]
        }
    )
    logger.info("unused item locks deleted count={}".format(result.deleted_count))


def is_locked(task):
    """Get info if task is locked."""
    lock_info = _lock.get_lock_info(task)
    return not (
        not lock_info or not lock_info["locked"] or (lock_info["expire"] is not None and lock_info["expire"] < utcnow())
    )


def touch(task, host=None, expire=1):
    """Touch lock on given task.

    It will extend expiry so task can run longer if needed.
    """
    if not host:
        host = get_host()
    try:
        _lock.touch(task, host, expire)
        return True
    except MongoLockException:
        return False



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : tomas
# Creation: 2018-11-14 15:37

from superdesk.commands.data_updates import BaseDataUpdate


def get_root_nodes(tree_items):
    root_nodes = []

    for key in tree_items:
        node = tree_items[key]
        if node.parent is None:
            root_nodes.append(node)

    return root_nodes


def get_ids_recursive(list_of_nodes):
    ids = []

    for node in list_of_nodes:
        if len(node.children) > 0:
            ids.extend(get_ids_recursive(node.children))

        ids.append(node.id)

    return ids


class TreeNode:
    def __init__(self, id):
        self.id = id
        self.parent = None
        self.children = []


class DataUpdate(BaseDataUpdate):

    resource = "archive"

    def forwards(self, mongodb_collection, mongodb_database):

        # building multiple trees
        tree_items = {}
        for item in mongodb_collection.find({"translated_from": {"$exists": True}}):
            node_id = item["_id"]

            if node_id not in tree_items:
                tree_items[node_id] = TreeNode(node_id)

            node = tree_items[node_id]

            parent_id = item["translated_from"]

            if parent_id not in tree_items:
                tree_items[parent_id] = TreeNode(parent_id)

            node.parent = tree_items[parent_id]
            node.parent.children.append(node)

        # processing trees
        for root_node in get_root_nodes(tree_items):
            ids = get_ids_recursive([root_node])

            print(mongodb_collection.update_many({"_id": {"$in": ids}}, {"$set": {"translation_id": root_node.id}}))

    def backwards(self, mongodb_collection, mongodb_database):
        raise NotImplementedError()



import io
import bson
import flask
import unittest

from unittest.mock import create_autospec, patch
from superdesk.storage.proxy import ProxyMediaStorage
from superdesk.storage.desk_media_storage import SuperdeskGridFSMediaStorage
from superdesk.storage.amazon_media_storage import AmazonMediaStorage


class SuperdeskMediaStorageTestCase(unittest.TestCase):
    def setUp(self):
        self.app = flask.Flask(__name__)
        self.app.config.update(
            dict(
                AMAZON_ACCESS_KEY_ID="foo",
                AMAZON_SECRET_ACCESS_KEY="bar",
                AMAZON_ENDPOINT_URL="http://example.com",
            )
        )
        self.storage = ProxyMediaStorage(self.app)
        self.storage._storage = [
            create_autospec(SuperdeskGridFSMediaStorage),
            create_autospec(AmazonMediaStorage),
        ]

    def test_storage(self):
        content = io.BytesIO(b"foo")
        _media = {"_id": "test"}

        self.storage._storage[0].exists.return_value = False
        self.storage._storage[0].get_by_filename.return_value = "mongo"

        self.storage._storage[1].exists.return_value = True
        self.storage._storage[1].get.return_value = _media
        self.storage._storage[1].url_for_media.return_value = "url_for_media"
        self.storage._storage[1].url_for_download.return_value = "url_for_download"

        self.storage._storage[0].put.return_value = "media_id"
        _id = self.storage.put(content, "filename", "text/plain", "test")
        self.storage._storage[0].put.assert_called_once_with(
            content, filename="filename", content_type="text/plain", metadata="test", resource=None
        )

        self.assertIsNotNone(self.storage.get(_id))

        self.storage.delete(_id)
        self.storage._storage[1].delete.assert_called_once_with(_id, None)

        self.assertEqual(_media, self.storage.fetch_rendition({"media": "foo"}))

        self.assertEqual("url_for_media", self.storage.url_for_media("test"))
        self.assertEqual("url_for_download", self.storage.url_for_download("test"))

        self.assertEqual("mongo", self.storage.get_by_filename("path"))

    def test_amazon_get_by_filename_mongo(self):
        _id = bson.ObjectId()
        amazon = AmazonMediaStorage(self.app)
        with patch.object(amazon, "get") as get_mock:
            amazon.get_by_filename("{}.jpg".format(str(_id)))
            get_mock.assert_called_once_with(str(_id))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import ftplib
import logging
import tempfile
from urllib.parse import urlparse
from datetime import datetime

from flask import current_app as app
from superdesk.io.registry import register_feeding_service
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.utc import utc
from superdesk.etree import etree
from superdesk.io.feeding_services import FeedingService
from superdesk.errors import IngestFtpError
from superdesk.ftp import ftp_connect


logger = logging.getLogger(__name__)
DEFAULT_SUCCESS_PATH = "_PROCESSED"
DEFAULT_FAILURE_PATH = "_ERROR"


class EmptyFile(Exception):
    """Raised when a file is empty thus ignored"""


class FTPFeedingService(FeedingService):
    """
    Feeding Service class which can read article(s) which exist in a file system and accessible using FTP.
    """

    NAME = "ftp"

    ERRORS = [
        IngestFtpError.ftpUnknownParserError().get_error_description(),
        IngestFtpError.ftpError().get_error_description(),
    ]

    label = "FTP feed"

    fields = [
        {
            "id": "host",
            "type": "text",
            "label": "Host",
            "placeholder": "FTP Server URL",
            "required": True,
            "errors": {5003: "Server not found."},
        },
        {
            "id": "username",
            "type": "text",
            "label": "Username",
            "placeholder": "Username",
            "required": False,
            "errors": {5002: "Credentials error."},
        },
        {"id": "password", "type": "password", "label": "Password", "placeholder": "Password", "required": False},
        {"id": "path", "type": "text", "label": "Path", "placeholder": "FTP Server Path", "required": False},
        {"id": "dest_path", "type": "text", "label": "Local Path", "placeholder": "Local Path", "required": True},
        {
            "id": "passive",
            "type": "boolean",
            "label": "Passive",
            "placeholder": "Passive",
            "required": False,
            "default": True,
        },
        {
            "id": "move",
            "type": "boolean",
            "label": "Move items after ingestion",
            "placeholder": "Move items after ingestion",
            "required": False,
        },
        {
            "id": "ftp_move_path",
            "type": "text",
            "label": "Move ingested items to",
            "placeholder": "FTP Server Path, keep empty to use default path",
            "required": False,
            "show_expression": "{move}",
        },
        {
            "id": "move_path_error",
            "type": "text",
            "label": "Move *NOT* ingested items (i.e. on error) to",
            "placeholder": "FTP Server Path, keep empty to use default path",
            "required": False,
            "show_expression": "{move}",
        },
    ]

    DATE_FORMAT = "%Y%m%d%H%M%S"

    ALLOWED_EXT_DEFAULT = {".json", ".xml"}

    def config_from_url(self, url):
        """
        Parse given url into ftp config.

        :param url: url in form `ftp://username:password@host:port/dir`
        """
        url_parts = urlparse(url)
        return {
            "username": url_parts.username,
            "password": url_parts.password,
            "host": url_parts.hostname,
            "path": url_parts.path.lstrip("/"),
        }

    def _test(self, provider):
        config = provider.get("config", {})
        try:
            with ftp_connect(config) as ftp:
                ftp.mlsd()
        except IngestFtpError:
            raise
        except Exception as ex:
            if "500" in str(ex):
                ftp.nlst()
            else:
                raise IngestFtpError.ftpError(ex, provider)

    def _move(self, ftp, src, dest, file_modify, failed):
        """Move distant file

        file won't be moved if it is failed and last modification was made
        recently enough (i.e. before config's INGEST_OLD_CONTENT_MINUTES is
        expired). In other words, if a file fails, it will be tried again until
        INGEST_OLD_CONTENT_MINUTES delay expires.

        :param ftp: FTP instance to use
        :type ftp: ftplib.FTP
        :param src: source path of the file to move
        :type src: str
        :param dest: dest path of the file to move
        :type dest: str
        :param file_modify: date of last file modification
        :type file_modify: datetime
        :param failed: True if something when wrong during ingestion
        :type failed: bool
        """
        if failed and not self.is_old_content(file_modify):
            logger.warning(
                "{src!r} ingestion failed, but we are in the backstop delay, it will be "
                "tried again next time".format(src=src)
            )
            return
        try:
            ftp.rename(src, dest)
        except ftplib.all_errors as e:
            logger.warning("Can't move file from {src} to {dest}: {reason}".format(src=src, dest=dest, reason=e))

    def _create_if_missing(self, ftp, path):
        """Check if a dir exists, and create it else

        :param ftp: FTP instance to use
        :type ftp: ftplib.FTP
        :param src: dir path to check
        :type src: str
        """
        base_path = ftp.pwd()
        try:
            ftp.cwd(path)
        except ftplib.all_errors:
            # path probably doesn't exist
            # catching all_errors is a bit overkill,
            # but ftplib doesn't really have precise error
            # for missing directory
            if path.startswith("./"):
                ftp.cwd("/")
                ftp.mkd(path)
            elif not path.startswith("/"):
                ftp.mkd("/" + path)
            else:
                ftp.mkd(path)
        finally:
            ftp.cwd(base_path)

    def _create_move_folders(self, config, ftp):
        if not config.get("ftp_move_path"):
            logger.debug("missing move_path, default will be used")
        move_path = os.path.join(config.get("path", ""), config.get("ftp_move_path") or DEFAULT_SUCCESS_PATH)

        if not config.get("move_path_error"):
            logger.debug("missing move_path_error, default will be used")
        move_path_error = os.path.join(config.get("path", ""), config.get("move_path_error") or DEFAULT_FAILURE_PATH)

        try:
            self._create_if_missing(ftp, move_path)
            self._create_if_missing(ftp, move_path_error)
        except ftplib.all_errors as e:
            logger.error("Can't create move directory: {reason}".format(reason=e))
            raise e

        return move_path, move_path_error

    def _is_allowed(self, filename, allowed_ext):
        """Test if given file is allowed to be ingested."""
        _, ext = os.path.splitext(filename)
        return ext.lower() in allowed_ext

    def _is_empty(self, file_path):
        """Test if given file path is empty, return True if a file is empty"""
        return not (os.path.isfile(file_path) and os.path.getsize(file_path) > 0)

    def _list_files(self, ftp, provider):
        self._timer.start("ftp_list")
        try:
            return [(filename, facts["modify"]) for filename, facts in ftp.mlsd() if facts.get("type") == "file"]
        except Exception as ex:
            if "500" in str(ex):
                file_list = []
                file_name_list = []
                date_list = []
                ftp.dir(file_list.append)
                self.DATE_FORMAT = "%Y %b %d %H:%M:%S"
                for line in file_list:
                    col = line.split()
                    date_string = "{} ".format(datetime.now().year) + " ".join(col[5:8]) + ":00"
                    date_list.append(date_string)
                    file_name_list.append(col[8])
                return zip(file_name_list, date_list)
            else:
                raise IngestFtpError.ftpError(ex, provider)
        finally:
            self._log_msg("FTP list files. Exec time: {:.4f} secs.".format(self._timer.stop("ftp_list")))

    def _sort_files(self, files):
        self._timer.start("sort_files")
        files = sorted(files, key=lambda x: x[1])
        self._log_msg("Sort {} files. Exec time: {:.4f} secs.".format(len(files), self._timer.stop("sort_files")))
        return files

    def _retrieve_and_parse(self, ftp, config, filename, provider, registered_parser):
        self._timer.start("retrieve_parse")

        if "dest_path" not in config:
            config["dest_path"] = tempfile.mkdtemp(prefix="superdesk_ingest_")
        local_file_path = os.path.join(config["dest_path"], filename)

        with open(local_file_path, "wb") as f:
            try:
                ftp.retrbinary("RETR %s" % filename, f.write)
                self._log_msg(
                    "Download finished. Exec time: {:.4f} secs. Size: {} bytes. File: {}.".format(
                        self._timer.split("retrieve_parse"), os.path.getsize(local_file_path), filename
                    )
                )
            except ftplib.all_errors:
                self._log_msg(
                    "Download failed. Exec time: {:.4f} secs. File: {}.".format(
                        self._timer.stop("retrieve_parse"), filename
                    )
                )
                os.remove(local_file_path)
                raise Exception("Exception retrieving file from FTP server ({filename})".format(filename=filename))

        if self._is_empty(local_file_path):
            logger.info("ignoring empty file {filename}".format(filename=filename))
            raise EmptyFile(local_file_path)

        if isinstance(registered_parser, XMLFeedParser):
            xml = etree.parse(local_file_path).getroot()
            parser = self.get_feed_parser(provider, xml)
            parsed = parser.parse(xml, provider)
        else:
            parser = self.get_feed_parser(provider, local_file_path)
            parsed = parser.parse(local_file_path, provider)

        self._log_msg(
            "Parsing finished. Exec time: {:.4f} secs. File: {}.".format(self._timer.stop("retrieve_parse"), filename)
        )

        return [parsed] if isinstance(parsed, dict) else parsed

    def _update(self, provider, update):
        config = provider.get("config", {})
        do_move = config.get("move", False)
        last_processed_file_modify = provider.get("private", {}).get("last_processed_file_modify")
        limit = app.config.get("FTP_INGEST_FILES_LIST_LIMIT", 100)
        registered_parser = self.get_feed_parser(provider)
        allowed_ext = getattr(registered_parser, "ALLOWED_EXT", self.ALLOWED_EXT_DEFAULT)

        try:
            self._timer.start("ftp_connect")
            with ftp_connect(config) as ftp:
                self._log_msg(
                    "Connected to FTP server. Exec time: {:.4f} secs.".format(self._timer.stop("ftp_connect"))
                )
                files_to_process = []
                files = self._sort_files(self._list_files(ftp, provider))

                if do_move:
                    move_path, move_path_error = self._create_move_folders(config, ftp)

                self._timer.start("files_to_process")

                for filename, modify in files:
                    # filter by extension
                    if not self._is_allowed(filename, allowed_ext):
                        logger.info("ignoring file {filename} because of file extension".format(filename=filename))
                        continue

                    # filter by modify datetime
                    file_modify = datetime.strptime(modify, self.DATE_FORMAT).replace(tzinfo=utc)
                    if last_processed_file_modify:
                        # ignore limit and add files for processing
                        if last_processed_file_modify == file_modify:
                            files_to_process.append((filename, file_modify))
                        elif last_processed_file_modify < file_modify:
                            # even if we have reached a limit, we must add at least one file to increment
                            # a `last_processed_file_modify` in provider
                            files_to_process.append((filename, file_modify))
                            # limit amount of files to process per ingest update
                            if len(files_to_process) >= limit:
                                break
                    else:
                        # limit amount of files to process per ingest update
                        if len(files_to_process) >= limit:
                            break
                        # add files for processing
                        files_to_process.append((filename, file_modify))

                self._log_msg(
                    "Got {} file for processing. Exec time: {:.4f} secs.".format(
                        len(files_to_process), self._timer.stop("files_to_process")
                    )
                )

                # process files
                self._timer.start("start_processing")
                for filename, file_modify in files_to_process:
                    try:
                        update["private"] = {"last_processed_file_modify": file_modify}
                        failed = yield self._retrieve_and_parse(ftp, config, filename, provider, registered_parser)

                        if do_move:
                            move_dest_file_path = os.path.join(move_path if not failed else move_path_error, filename)
                            self._move(ftp, filename, move_dest_file_path, file_modify, failed=failed)
                    except EmptyFile:
                        continue
                    except Exception as e:
                        logger.error("Error while parsing {filename}: {msg}".format(filename=filename, msg=e))

                        if do_move:
                            move_dest_file_path_error = os.path.join(move_path_error, filename)
                            self._move(ftp, filename, move_dest_file_path_error, file_modify, failed=True)

                self._log_msg(
                    "Processing finished. Exec time: {:.4f} secs.".format(self._timer.stop("start_processing"))
                )

        except IngestFtpError:
            raise
        except Exception as ex:
            raise IngestFtpError.ftpError(ex, provider)


register_feeding_service(FTPFeedingService)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from unittest import mock
from superdesk.tests import TestCase
from superdesk.io.feeding_services import http_base_service
from superdesk.io.feeding_services import ap
from superdesk.io.feed_parsers import newsml_2_0
from copy import deepcopy

PREFIX = "test_superdesk_"
PROVIDER = {
    "_id": "test_provider",
    "config": {
        "username": "user",
        "password": "password",
        "idList": "123",
        "feed_parser": "newsml2",
        "field_aliases": [],
    },
}


class APTestCase(TestCase):
    def setUp(self):
        super().setUp()
        with self.app.app_context():
            vocab = [{}]
            self.app.data.insert("vocabularies", vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", "ap.xml"))
        with open(fixture, "rb") as f:
            self.feed_raw = f.read()

    @mock.patch.object(http_base_service, "requests")
    @mock.patch.object(ap.APFeedingService, "get_feed_parser")
    def test_feeding(self, get_feed_parser, requests):
        get_feed_parser.return_value = newsml_2_0.NewsMLTwoFeedParser()
        mock_get = requests.get.return_value
        mock_get.content = self.feed_raw
        provider = deepcopy(PROVIDER)
        service = ap.APFeedingService()
        service.provider = provider
        items = service._update(provider, {})[0]
        self.assertEqual(len(items), 3)

    @mock.patch.object(http_base_service, "requests")
    @mock.patch.object(ap.APFeedingService, "get_feed_parser")
    def test_items_order(self, get_feed_parser, requests):
        """Test that items are reversed on first call (SDESK-4372)

        Items of a new provider must be in reverse chronological order
        while further updates should give chronological order
        so we check that "reverse" has been called on items to fix order
        on first call.
        """
        feed_parser = newsml_2_0.NewsMLTwoFeedParser()
        get_feed_parser.return_value = feed_parser
        mock_get = requests.get.return_value
        mock_get.content = self.feed_raw
        provider = deepcopy(PROVIDER)
        service = ap.APFeedingService()
        service.provider = provider

        self.assertNotIn("private", provider)
        with mock.patch.object(feed_parser, "parse"):
            update = {}
            items = service._update(provider, update)[0]
            items.reverse.assert_called_once_with()
            provider.update(update)

        # because the provider has been run at least one time,
        # private data must now be present
        self.assertIn("private", provider)
        with mock.patch.object(feed_parser, "parse"):
            items = service._update(provider, {})[0]
            items.reverse.assert_not_called()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import datetime

from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.errors import ParserError
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE
from superdesk.utc import utc


class WENNFeedParser(XMLFeedParser):
    """
    Feed Parser for parsing the XML supplied by WENN
    """

    NAME = "wenn"

    label = "WENN Parser"

    ATOM_NS = "http://www.w3.org/2005/Atom"
    WENN_MM_NS = "http://feed.wenn.com/xmlns/2010/03/wenn_mm"
    WENN_NM_NS = "http://feed.wenn.com/xmlns/2010/03/wenn_nm"
    WENN_CM_NS = "http://feed.wenn.com/xmlns/2010/03/wenn_cm"
    GEORSS_NS = "http://www.georss.org/georss"

    def can_parse(self, xml):
        return (
            xml.tag == self.qname("feed", self.ATOM_NS)
            and len(xml.findall(self.qname("NewsManagement", self.WENN_NM_NS))) > 0
        )

    def parse(self, xml, provider=None):
        itemList = []
        try:
            for entry in xml.findall(self.qname("entry", self.ATOM_NS)):
                item = {}
                self.set_item_defaults(item)
                self.parse_content_management(item, entry)
                self.parse_news_management(item, entry)
                item["body_html"] = self.get_elem_content(entry.find(self.qname("content", self.ATOM_NS)))
                item["body_html"] = item["body_html"].replace("\n\n  ", "</p><p>").replace("\n", "<br>")
                item["body_html"] = "<p>" + item["body_html"] + "</p>"
                itemList.append(item)
            return itemList

        except Exception as ex:
            raise ParserError.wennParserError(ex, provider)

    def set_item_defaults(self, item):
        item[ITEM_TYPE] = CONTENT_TYPE.TEXT
        item["urgency"] = 5
        item["pubstatus"] = "usable"
        item["anpa_category"] = [{"qcode": "e"}]
        item["subject"] = [{"qcode": "01000000", "name": "arts, culture and entertainment"}]

    def parse_news_management(self, item, entry):
        news_mgmt_el = entry.find(self.qname("NewsManagement", self.WENN_NM_NS))
        if news_mgmt_el is not None:
            item["firstcreated"] = self.datetime(
                self.get_elem_content(news_mgmt_el.find(self.qname("published", self.WENN_NM_NS)))
            )
            item["versioncreated"] = self.datetime(
                self.get_elem_content(news_mgmt_el.find(self.qname("updated", self.WENN_NM_NS)))
            )
            item["guid"] = self.get_elem_content(news_mgmt_el.find(self.qname("original_article_id", self.WENN_NM_NS)))

    def parse_content_management(self, item, entry):
        content_mgmt_el = entry.find(self.qname("ContentMetadata", self.WENN_CM_NS))
        if content_mgmt_el is not None:
            item["headline"] = self.get_elem_content(content_mgmt_el.find(self.qname("title", self.WENN_CM_NS)))
            item["abstract"] = self.get_elem_content(content_mgmt_el.find(self.qname("first_line", self.WENN_CM_NS)))

    def get_elem_content(self, elem):
        return elem.text if elem is not None else ""

    def datetime(self, string):
        return datetime.datetime.strptime(string, "%Y-%m-%dT%H:%M:%S+00:00").replace(tzinfo=utc)


register_feed_parser(WENNFeedParser.NAME, WENNFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from superdesk import get_backend
import superdesk

from apps.publish.content import (
    ArchivePublishResource,
    ArchivePublishService,
    KillPublishResource,
    KillPublishService,
    CorrectPublishResource,
    CorrectPublishService,
    ResendResource,
    ResendService,
    TakeDownPublishService,
    TakeDownPublishResource,
    UnpublishResource,
    UnpublishService,
)
from apps.publish.enqueue import EnqueueContent
from apps.publish.published_item import PublishedItemResource, PublishedItemService
from apps.publish.content.published_package_items import PublishedPackageItemsService, PublishedPackageItemsResource


logger = logging.getLogger(__name__)


def init_app(app):

    endpoint_name = "archive_publish"
    service = ArchivePublishService(endpoint_name, backend=get_backend())
    ArchivePublishResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_kill"
    service = KillPublishService(endpoint_name, backend=get_backend())
    KillPublishResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_correct"
    service = CorrectPublishService(endpoint_name, backend=get_backend())
    CorrectPublishResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_takedown"
    service = TakeDownPublishService(endpoint_name, backend=get_backend())
    TakeDownPublishResource(endpoint_name, app=app, service=service)

    endpoint_name = "published"
    service = PublishedItemService(endpoint_name, backend=get_backend())
    PublishedItemResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_resend"
    service = ResendService(endpoint_name, backend=get_backend())
    ResendResource(endpoint_name, app=app, service=service)

    endpoint_name = "published_package_items"
    service = PublishedPackageItemsService(endpoint_name, backend=get_backend())
    PublishedPackageItemsResource(endpoint_name, app=app, service=service)

    endpoint_name = "archive_unpublish"
    service = UnpublishService(endpoint_name, backend=get_backend())
    UnpublishResource(endpoint_name, app=app, service=service)

    superdesk.privilege(name="subscribers", label="Subscribers", description="User can manage subscribers")
    superdesk.privilege(name="publish", label="Publish", description="Publish a content")
    superdesk.privilege(name="kill", label="Kill", description="Kill a published content")
    superdesk.privilege(name="correct", label="Correction", description="Correction to a published content")
    superdesk.privilege(name="publish_queue", label="Publish Queue", description="User can update publish queue")
    superdesk.privilege(name="resend", label="Resending Stories", description="User can resend published stories")
    superdesk.privilege(name="embargo", label="Embargo", description="User can set embargo date")
    superdesk.privilege(name="takedown", label="Take down", description="Take down a published content")
    superdesk.privilege(name="unpublish", label="Unpublish", description="Unpublish a published content")


def enqueue_content():
    EnqueueContent().run()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource


class MarkedForDesksResource(Resource):
    """Marked for desks Schema"""

    schema = {"marked_desk": {"type": "string", "required": True}, "marked_item": {"type": "string", "required": True}}
    privileges = {"POST": "mark_for_desks"}



import unittest
import superdesk.utils as utils


class UtilsTestCase(unittest.TestCase):
    def test_sha(self):
        digest = utils.sha("some text")
        self.assertGreater(len(digest), 40)

    def test_get_random_token(self):
        token = utils.get_random_token()
        self.assertGreater(len(token), 20)
        self.assertNotEqual(token, utils.get_random_token())

    def test_save_error_data(self):
        filename = utils.save_error_data("foo")
        self.assertIn("superdesk", filename)
        with open(filename) as f:
            self.assertEqual("foo", f.read())

    def test_timer(self):
        timer = utils.Timer()

        timer.start("test")
        self.assertIsNotNone(timer.split("test"))
        self.assertIsNotNone(timer.split("test"))
        self.assertIsNotNone(timer.stop("test"))
        self.assertRaises(KeyError, timer.split, "test")
        self.assertRaises(KeyError, timer.split, "bla")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
import os
from superdesk.locators.locators import LocatorIndex, bp
import superdesk


def _load_json(file_path):
    """
    Reads JSON string from the file located in file_path.

    :param file_path: path of the file having JSON string.
    :return: JSON Object
    """
    with open(file_path, "r", encoding="utf-8") as f:
        return json.load(f)


def init_app(app):
    app.locators = LocatorIndex()
    superdesk.blueprint(bp, app)
    _locators_file_path = app.config.get(
        "LOCATORS_DATA_FILE", os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "locators.json")
    )
    app.locators.register(_load_json(_locators_file_path))



import superdesk

from flask import current_app as app
from superdesk.utils import ListCursor


class ClientConfigResource(superdesk.Resource):
    item_methods = []
    public_methods = ["GET"]
    resource_methods = ["GET"]


class ClientConfigService(superdesk.Service):
    def get(self, req, lookup):
        return ListCursor()

    def on_fetched(self, docs):
        docs["config"] = getattr(app, "client_config", {})


def init_app(app):
    superdesk.register_resource("client_config", ClientConfigResource, ClientConfigService, _app=app)
    app.client_config.update(
        {
            "schema": app.config.get("SCHEMA"),
            "editor": app.config.get("EDITOR"),
            "feedback_url": app.config.get("FEEDBACK_URL"),
            "override_ednote_for_corrections": app.config.get("OVERRIDE_EDNOTE_FOR_CORRECTIONS", True),
            "override_ednote_template": app.config.get("OVERRIDE_EDNOTE_TEMPLATE"),
            "default_genre": app.config.get("DEFAULT_GENRE_VALUE_FOR_MANUAL_ARTICLES"),
            "japanese_characters_per_minute": app.config.get("JAPANESE_CHARACTERS_PER_MINUTE"),
            "validator_media_metadata": app.config.get("VALIDATOR_MEDIA_METADATA", {}),
            "publish_content_expiry_minutes": app.config.get("PUBLISHED_CONTENT_EXPIRY_MINUTES", 0),
            "high_priority_queue_enabled": app.config.get("HIGH_PRIORITY_QUEUE_ENABLED"),
            "default_language": app.config.get("DEFAULT_LANGUAGE"),
            "workflow_allow_multiple_updates": app.config.get("WORKFLOW_ALLOW_MULTIPLE_UPDATES"),
            "workflow_allow_duplicate_non_members": app.config.get("WORKFLOW_ALLOW_DUPLICATE_NON_MEMBERS"),
            "disallowed_characters": app.config.get("DISALLOWED_CHARACTERS"),
            "allow_updating_scheduled_items": app.config.get("ALLOW_UPDATING_SCHEDULED_ITEMS"),
            "corrections_workflow": app.config.get("CORRECTIONS_WORKFLOW"),
        }
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import json

from eve.utils import ParsedRequest

import superdesk
import logging
from copy import deepcopy
from flask import current_app as app
from superdesk import get_resource_service, config
from superdesk.errors import SuperdeskApiError
from superdesk.media.media_operations import crop_image, process_file_from_stream
from superdesk.upload import url_for_media
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE, MEDIA_TYPES, ASSOCIATIONS
from .renditions import _resize_image


logger = logging.getLogger(__name__)


class CropService:

    crop_sizes = []

    def validate_crop(self, original, updates, crop_name):
        """Validate crop info on media item.

        :param dict original: original item
        :param dict updates: updated renditions
        :param str crop_name: name of the crop
        :param dict doc: crop co-ordinates
        :raises SuperdeskApiError.badRequestError:
            For following conditions:
            1) if type != picture
            2) if renditions are missing in the original image
            3) if original rendition is missing
            4) Crop name is invalid
        """
        # Check if type is picture
        if original[ITEM_TYPE] != CONTENT_TYPE.PICTURE:
            raise SuperdeskApiError.badRequestError(message="Only images can be cropped!")

        # Check if the renditions exists
        if not original.get("renditions"):
            raise SuperdeskApiError.badRequestError(message="Missing renditions!")

        # Check if the original rendition exists
        if not original.get("renditions").get("original"):
            raise SuperdeskApiError.badRequestError(message="Missing original rendition!")

        # Check if the crop name is valid
        crop = self.get_crop_by_name(crop_name)
        crop_data = updates.get("renditions", {}).get(crop_name, {})
        if not crop and "CropLeft" in crop_data:
            raise SuperdeskApiError.badRequestError(message="Unknown crop name! (name=%s)" % crop_name)

        self._validate_values(crop_data)
        self._validate_poi(original, updates, crop_name)
        self._validate_aspect_ratio(crop, crop_data)

    def _validate_values(self, crop):
        int_fields = ("CropLeft", "CropTop", "CropRight", "CropBottom", "width", "height")
        for field in int_fields:
            if field in crop:
                try:
                    crop[field] = int(crop[field])
                except (TypeError, ValueError):
                    raise SuperdeskApiError.badRequestError("Invalid value for %s in renditions" % field)

    def _validate_poi(self, original, updates, crop_name):
        """Validate the crop point of interest in the renditions dictionary for the given crop

        :param dict original: original item
        :param dict updates: updated renditions
        """
        renditions = original.get("renditions", {})
        updated_renditions = updates.get("renditions", {})
        original_image = deepcopy(renditions["original"])
        original_image.update(updated_renditions.get("original", {}))
        if "poi" in updates:
            if "x" not in updates["poi"] or "y" not in updates["poi"]:
                del updates["poi"]
                return
            poi = updates["poi"]
        elif "poi" not in original:
            return
        else:
            if crop_name not in updated_renditions:
                return
            poi = original["poi"]

        crop_data = updated_renditions[crop_name] if crop_name in updated_renditions else renditions[crop_name]
        orig_poi_x = int(original_image["width"] * poi["x"])
        orig_poi_y = int(original_image["height"] * poi["y"])

        if (
            orig_poi_y < crop_data.get("CropTop", 0)
            or orig_poi_y > crop_data.get("CropBottom", original_image["height"])
            or orig_poi_x < crop_data.get("CropLeft", 0)
            or orig_poi_x > crop_data.get("CropRight", original_image["width"])
        ):
            raise SuperdeskApiError("Point of interest outside the crop %s limits" % crop_name)

    def _validate_aspect_ratio(self, crop, doc):
        """Checks if the aspect ratio is consistent with one in defined in spec

        :param crop: Spec parameters
        :param doc: Posted parameters
        :raises SuperdeskApiError.badRequestError:
        """
        if "CropLeft" not in doc:
            return

        width = doc["CropRight"] - doc["CropLeft"]
        height = doc["CropBottom"] - doc["CropTop"]
        if not (crop.get("width") or crop.get("height") or crop.get("ratio")):
            raise SuperdeskApiError.badRequestError(
                message="Crop data are missing. width, height or ratio need to be defined"
            )
        if crop.get("width") and crop.get("height"):
            expected_crop_width = int(crop["width"])
            expected_crop_height = int(crop["height"])
            if width < expected_crop_width or height < expected_crop_height:
                raise SuperdeskApiError.badRequestError(
                    message="Wrong crop size. Minimum crop size is {}x{}.".format(crop["width"], crop["height"])
                )
                doc_ratio = round(width / height, 1)
                spec_ratio = round(expected_crop_width / expected_crop_height, 1)
                if doc_ratio != spec_ratio:
                    raise SuperdeskApiError.badRequestError(message="Wrong aspect ratio!")
        elif crop.get("ratio"):
            ratio = crop.get("ratio")
            if type(ratio) not in [int, float]:
                ratio = ratio.split(":")
                ratio = int(ratio[0]) / int(ratio[1])
            if abs((width / height) - ratio) > 0.1:
                raise SuperdeskApiError.badRequestError(
                    message="Ratio %s is not respected. We got %f" % (crop.get("ratio"), abs((width / height)))
                )

    def get_crop_by_name(self, crop_name):
        """Finds the crop in the list of crops by name

        :param crop_name: Crop name
        :return: Matching crop or None
        """
        if not self.crop_sizes:
            self.crop_sizes = get_resource_service("vocabularies").find_one(req=None, _id="crop_sizes").get("items")

        if not self.crop_sizes:
            raise SuperdeskApiError.badRequestError(message="Crops sizes couldn't be loaded!")

        return next((c for c in self.crop_sizes if c.get("name", "").lower() == crop_name.lower()), None)

    def create_crop(self, original_image, crop_name, crop_data):
        """Create a new crop based on the crop co-ordinates

        :param original: Article to add the crop
        :param crop_name: Name of the crop
        :param doc: Crop details
        :raises SuperdeskApiError.badRequestError
        :return dict: rendition
        """
        original_file = app.media.fetch_rendition(original_image)
        if not original_file:
            raise SuperdeskApiError.badRequestError("Original file couldn't be found")
        try:
            cropped, out = crop_image(original_file, crop_name, crop_data)
            crop = self.get_crop_by_name(crop_name)
            if not cropped:
                raise SuperdeskApiError.badRequestError("Saving crop failed.")
            # resize if needed
            if crop.get("width") or crop.get("height"):
                out, width, height = _resize_image(
                    out,
                    size=(crop.get("width"), crop.get("height")),
                    keepProportions=crop.get("keep_proportions", True),
                )
                crop["width"] = width
                crop["height"] = height
                out.seek(0)
            return self._save_cropped_image(out, original_image, crop_data)
        except SuperdeskApiError:
            raise
        except Exception as ex:
            raise SuperdeskApiError.badRequestError("Generating crop failed: {}".format(str(ex)))

    def _save_cropped_image(self, file_stream, original, doc):
        """Saves the cropped image and returns the crop dictionary

        :param file_stream: cropped image stream
        :param original: original rendition
        :param doc: crop data
        :return dict: Crop values
        :raises SuperdeskApiError.internalError
        """
        crop = {}
        try:
            file_name, content_type, metadata = process_file_from_stream(
                file_stream, content_type=original.get("mimetype")
            )
            file_stream.seek(0)
            file_id = app.media.put(
                file_stream, filename=file_name, content_type=content_type, resource="upload", metadata=metadata
            )
            crop["media"] = file_id
            crop["mimetype"] = content_type
            crop["href"] = url_for_media(file_id, content_type)
            crop["CropTop"] = doc.get("CropTop", None)
            crop["CropLeft"] = doc.get("CropLeft", None)
            crop["CropRight"] = doc.get("CropRight", None)
            crop["CropBottom"] = doc.get("CropBottom", None)
            return crop
        except Exception as ex:
            try:
                app.media.delete(file_id)
            except Exception:
                pass
            raise SuperdeskApiError.internalError("Generating crop failed: {}".format(str(ex)), exception=ex)

    def _delete_crop_file(self, file_id):
        """Delete the crop file

        :param Object_id file_id: Object_Id of the file.
        """
        try:
            app.media.delete(file_id)
        except Exception:
            logger.exception("Crop File cannot be deleted. File_Id {}".format(file_id))

    def create_multiple_crops(self, updates, original):
        """Create multiple crops based on the renditions.

        :param dict updates: update item
        :param dict original: original of the updated item
        """
        if original.get(ITEM_TYPE) != CONTENT_TYPE.PICTURE:
            return

        update_renditions = updates.get("renditions", {})
        renditions = deepcopy(original.get("renditions", {}))
        # keep renditions updates (urls may have changed)
        renditions.update(update_renditions)
        renditions = {k: renditions[k] for k in renditions if renditions[k]}

        if "original" in updates.get("renditions", {}):
            original_image = updates["renditions"]["original"]
        else:
            try:
                original_image = original["renditions"]["original"]
            except KeyError:
                return

        for key in [k for k in update_renditions if update_renditions[k]]:
            if not self.get_crop_by_name(key):
                continue

            original_crop = original.get("renditions", {}).get(key, {})
            fields = ("CropLeft", "CropTop", "CropRight", "CropBottom")
            crop_data = update_renditions.get(key, {})
            if any(crop_data.get(name) != original_crop.get(name) for name in fields) and not crop_data.get("media"):
                rendition = self.create_crop(original_image, key, crop_data)
                renditions[key] = rendition

        poi = updates.get("poi")
        if poi:
            for crop_name in renditions:
                self._set_crop_poi(renditions, crop_name, poi)

        updates["renditions"] = renditions

    def _set_crop_poi(self, renditions, crop_name, poi):
        """Set the crop point of interest in the renditions dictionary for the given crop

        :param dict renditions: updated renditions
        :param string crop_name: the crop for which to set the poi
        :param dict poi: the point of interest dictionary
        """
        fields = ("CropLeft", "CropTop", "CropRight", "CropBottom")
        if "x" in poi and "y" in poi:
            original_image = renditions["original"]
            crop_data = renditions[crop_name]
            orig_poi_x = int(original_image["width"] * poi["x"])
            orig_poi_y = int(original_image["height"] * poi["y"])

            if any(name in crop_data for name in fields):
                crop_poi_x = orig_poi_x - crop_data.get("CropLeft", 0)
                crop_poi_y = orig_poi_y - crop_data.get("CropTop", 0)
            else:
                crop_poi_x = int(crop_data.get("width", original_image["width"]) * poi["x"])
                crop_poi_y = int(crop_data.get("height", original_image["height"]) * poi["y"])
            renditions[crop_name]["poi"] = {"x": crop_poi_x, "y": crop_poi_y}

    def validate_multiple_crops(self, updates, original):
        """Validate crops for the image

        :param dict updates: update item
        :param dict original: original of the updated item
        """
        renditions = updates.get("renditions", {})
        if not (renditions and original.get(ITEM_TYPE) == CONTENT_TYPE.PICTURE):
            return

        for key in [k for k in renditions if renditions[k]]:
            self.validate_crop(original, updates, key)

    def delete_replaced_crop_files(self, updates, original):
        """Delete the replaced crop files.

        :param dict updates: update item
        :param dict original: original of the updated item
        """
        update_renditions = updates.get("renditions", {})
        if original.get(ITEM_TYPE) == CONTENT_TYPE.PICTURE and update_renditions:
            renditions = original.get("renditions", {})
            for key in update_renditions:
                if self.get_crop_by_name(key) and update_renditions.get(key, {}).get("media") != renditions.get(
                    key, {}
                ).get("media"):
                    self._delete_crop_file(renditions.get(key, {}).get("media"))

    def update_media_references(self, updates, original, published=False):
        """Update the media references collection.
         When item (media item or associated media) is updated or created,
         media_references are created. These media_references are updated to published state
         once the item is published.

        :param dict updates: Updates of the item
        :param dict original: Original item
        :param boolean published: True if publishing the item else False
        """
        item_id = original.get(config.ID_FIELD)
        references = {}
        if updates.get("renditions", original.get("renditions", {})):
            references = {item_id: updates.get("renditions", original.get("renditions", {}))}

        if original.get(ITEM_TYPE) not in MEDIA_TYPES:
            associations = updates.get(ASSOCIATIONS) or original.get(ASSOCIATIONS)
            if not associations:
                return

            references = {
                assoc.get(config.ID_FIELD): assoc.get("renditions")
                for assoc in associations.values()
                if assoc and assoc.get("renditions")
            }

        if not references:
            return

        for assoc_id, renditions in references.items():
            associated_id = assoc_id if assoc_id != item_id else None
            for rendition in [r for r in renditions.values() if r]:
                if not rendition.get("media"):
                    continue

                media = str(rendition.get("media"))
                reference = get_resource_service("media_references").find_one(req=None, item_id=item_id, media_id=media)
                if not reference:
                    try:
                        get_resource_service("media_references").post(
                            [
                                {
                                    "item_id": item_id,
                                    "media_id": media,
                                    "associated_id": associated_id,
                                    "published": False,
                                }
                            ]
                        )
                    except Exception:
                        logger.exception("Failed to insert media reference item {} media {}".format(item_id, media))

        # item is publish
        if not published:
            return

        req = ParsedRequest()
        req.where = json.dumps({"item_id": item_id, "published": False})
        refs = list(get_resource_service("media_references").get(req=req, lookup=None))
        for ref in refs:
            try:
                get_resource_service("media_references").patch(ref.get(config.ID_FIELD), updates={"published": True})
            except Exception:
                logger.exception(
                    "Failed to update media "
                    "reference item {} media {}".format(ref.get("item_id"), ref.get("media_id"))
                )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.feed_parsers.ritzau import RitzauFeedParser
from superdesk.tests import TestCase
from superdesk.etree import etree
import os


class BaseRitzauTestCase(TestCase):

    vocab = [{"_id": "genre", "items": [{"name": "Current"}]}]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert("vocabularies", self.vocab)

    def _parse_file(self, filename):
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", filename))
        provider = {"name": "Test"}
        with open(fixture, "rb") as f:
            self.root_elt = etree.fromstring(f.read())
            self.item = RitzauFeedParser().parse(self.root_elt, provider)


class RitzauTestCase(BaseRitzauTestCase):
    def test_can_parse(self):
        self._parse_file("ritzau_news.xml")
        self.assertTrue(RitzauFeedParser().can_parse(self.root_elt))

    def test_content(self):
        self._parse_file("ritzau_news.xml")
        item = self.item
        self.assertEqual(item["version"], 1)
        self.assertEqual(item["byline"], "/ritzau/")
        self.assertEqual(
            item["headline"],
            'Hollandske forskere har "i årevis" lavet forsøg på både mennesker og ' "dyr, hvor de har testet effekte",
        )
        self.assertEqual(
            item["body_html"],
            '<p>Hollandske forskere har "i årevis" lavet forsøg på både '
            "mennesker og dyr, hvor de har testet effekten af at indånde "
            "udstødningsgas fra dieselbiler.</p><p>Det fortæller forskerne "
            "selv til nyhedsbureauet AFP.</p><p>Historien kommer frem på et "
            "tidspunkt, hvor tyske bilfabrikanter er udsat for massiv "
            "kritik, fordi de har finansieret lignende forsøg.</p>",
        )
        self.assertEqual(item["guid"], "9a6955fc-11da-46b6-9903-439ebb288f2d")
        self.assertEqual(item["firstcreated"].isoformat(), "2018-01-30T16:32:18.397000+00:00")
        self.assertNotIn("ednote", item)
        self.assertEqual(item["priority"], 3)
        self.assertEqual(item["priority"], item["urgency"])

    def test_ednote(self):
        self._parse_file("ritzau_news_test_ednote.xml")
        self.assertEqual(
            self.item["ednote"],
            "Lasse Norman Hansen skifter til Corendon-Circus.\n"
            "Der i næste sæson kører på næsthøjeste niveau.\n"
            "Som led i en større omstrukturering skal den danske medicinalgigant "
            "Novo Nordisk sige farvel til omkring 400 ansatte inden for forskning og udvikling i Danmark og Kina.",
        )

    def test_cest_timezone(self):
        self.assertEqual(
            RitzauFeedParser()._publish_date_filter("2018-09-18T13:09:18.397").isoformat(),
            "2018-09-18T11:09:18.397000+00:00",
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_resource_service, editor_utils
from superdesk.media.crop import CropService
from superdesk.metadata.item import ITEM_STATE, EMBARGO, SCHEDULE_SETTINGS
from superdesk.utc import utcnow
from superdesk.text_utils import update_word_count
from apps.archive.common import set_sign_off, ITEM_OPERATION, get_user
from apps.archive.archive import flush_renditions
from apps.tasks import send_to
from apps.auth import get_user_id
from .common import BasePublishService, BasePublishResource, ITEM_CORRECT
from superdesk.emails import send_translation_changed
from superdesk.activity import add_activity
from flask import g, current_app as app


def send_translation_notifications(original):
    translated_items = get_resource_service("published").find(
        {"translation_id": original.get("guid"), "last_published_version": True}
    )

    user_ids = set()
    for translated_item in translated_items:
        if translated_item.get("item_id") == original.get("_id"):
            changed_article = translated_item
            continue
        user_ids.add(translated_item.get("original_creator"))
        user_ids.add(translated_item.get("version_creator"))

    if len(user_ids) == 0 or changed_article is None:
        return

    add_activity("translated:changed", "", resource=None, item=changed_article, notify=user_ids)

    recipients = []
    for user_id in user_ids:
        user = get_resource_service("users").find_one(req=None, _id=user_id)
        if user is not None and user.get("email", None) is not None:
            recipients.append(user.get("email"))

    if len(recipients) == 0:
        return

    username = g.user.get("display_name") or g.user.get("username")
    send_translation_changed(username, changed_article, recipients)


class CorrectPublishResource(BasePublishResource):
    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, "correct")


class CorrectPublishService(BasePublishService):
    publish_type = "correct"
    published_state = "corrected"
    item_operation = ITEM_CORRECT

    def set_state(self, original, updates):
        updates[ITEM_STATE] = self.published_state

        if original.get(EMBARGO) or updates.get(EMBARGO):
            # embargo time elapsed
            utc_embargo = updates.get(SCHEDULE_SETTINGS, {}).get("utc_{}".format(EMBARGO)) or original.get(
                SCHEDULE_SETTINGS, {}
            ).get("utc_{}".format(EMBARGO))
            if utc_embargo and utc_embargo < utcnow():
                # remove embargo information. so the next correction is without embargo.
                updates[EMBARGO] = None
                super().set_state(original, updates)
        else:
            super().set_state(original, updates)

    def change_being_corrected_to_published(self, updates, original):
        if app.config.get("CORRECTIONS_WORKFLOW") and original.get("state") == "correction":
            publish_service = get_resource_service("published")
            being_corrected_article = publish_service.find_one(
                req=None, guid=original.get("guid"), state="being_corrected"
            )

            if being_corrected_article.get("correction_sequence", 0) > 0:
                publish_service.patch(being_corrected_article["_id"], updates={"state": "corrected"})
            else:
                publish_service.patch(being_corrected_article["_id"], updates={"state": "published"})

    def send_to_original_desk(self, updates, original):
        if (
            app.config.get("CORRECTIONS_WORKFLOW")
            and original.get("state") == "correction"
            and original.get("task", {}).get("desk_history")
        ):
            send_to(
                doc=updates,
                desk_id=(original["task"]["desk_history"][0]),
                default_stage="working_stage",
                user_id=get_user_id(),
            )

    def on_update(self, updates, original):
        CropService().validate_multiple_crops(updates, original)
        super().on_update(updates, original)
        updates[ITEM_OPERATION] = self.item_operation
        updates["versioncreated"] = utcnow()
        updates["correction_sequence"] = original.get("correction_sequence", 1) + 1
        set_sign_off(updates, original)
        update_word_count(updates, original)
        flush_renditions(updates, original)
        self.change_being_corrected_to_published(updates, original)
        self.send_to_original_desk(updates, original)

    def update(self, id, updates, original):
        editor_utils.generate_fields(updates)
        get_resource_service("archive")._handle_media_updates(updates, original, get_user())
        super().update(id, updates, original)

    def on_updated(self, updates, original):
        super().on_updated(updates, original)
        send_translation_notifications(original)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk
import elasticsearch

from flask import current_app
from eve_elastic import get_es, get_indices, reindex
from superdesk.utils import get_random_string


class RebuildElasticIndex(superdesk.Command):
    """Rebuild the elastic indexes from existing data.

    It creates new index with same alias as the configured index,
    puts the new mapping and removes the old index.

    Example:
    ::

        $ python manage.py app:rebuild_elastic_index
        $ python manage.py app:rebuild_elastic_index --resource=items
        $ python manage.py app:rebuild_elastic_index --resource=archive

    """

    option_list = [
        superdesk.Option('--resource', '-r', dest='resource_name')
    ]

    def run(self, resource_name=None):
        # if no index name is passed then use the configured one
        resources = list(current_app.data.elastic._get_elastic_resources().keys())
        if resource_name and resource_name in resources:
            resources = [resource_name]
        elif resource_name:
            raise ValueError("Resource {} is not configured".format(resource_name))
        for resource in resources:
            try:
                current_app.data.elastic.reindex(resource)
            except elasticsearch.exceptions.NotFoundError as nfe:
                print(nfe)
            print('Index {} rebuilt successfully.'.format(resource_name))


superdesk.command('app:rebuild_elastic_index', RebuildElasticIndex())



import flask
import unittest
from superdesk.factory.sentry import SuperdeskSentry


class SentryTestCase(unittest.TestCase):
    def test_sentry_not_configured(self):
        app = flask.Flask(__name__)
        sentry = SuperdeskSentry(app)
        self.assertIsNone(sentry.captureMessage("test"))
        self.assertIsNone(sentry.captureException())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.auth import AuthResource
import superdesk
from superdesk.services import BaseService
from .auth import AuthUsersResource


def init_app(app):
    endpoint_name = "auth"
    service = BaseService("auth", backend=superdesk.get_backend())
    AuthResource(endpoint_name, app=app, service=service)

    service = BaseService("auth_user", backend=superdesk.get_backend())
    AuthUsersResource("auth_user", app=app, service=service)
